-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004,2006 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/_geteventviewerevents.sql, tbsm.sql, tbsm_base31e
-- SID: 1.36.4.21
-- Date Modified: 07/12/05 16:38:49
--
-- Module Description
--   Defines the _GetEventViewerEvents stored procedure

USE Object
GO

PRINT 'Creating procedure _GetEventViewerEvents'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='_GetEventViewerEvents')
  DROP PROCEDURE _GetEventViewerEvents
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004,2006 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/_geteventviewerevents.sql
-- SID: 1.36.4.21
-- Date Modified: 07/12/05 16:38:49
-- Implementation of the GetEventViewerEvents method
CREATE PROCEDURE _GetEventViewerEvents
  @cid                 ClassID,
  @id                  ObjID,
  @root_cid            ClassID = NULL,
  @root_id             ObjID = 0,
  @Filters             NVARCHAR(1000) = NULL,
  @OrderBy             NVARCHAR(1000) = NULL,
  @MaxRows             INT = 0,
  @GetChildren         TINYINT = 1,
  @Direction           TINYINT = 1,
  @mesg_id             ObjID = 0 OUTPUT,
  @excp_id             ObjID = 0 OUTPUT,
  @numevents           INT = 0 OUTPUT,
  @numfilteredevents   INT = 0 OUTPUT,
  @bForceReload        TINYINT = 0, 
  @OutputTable         sysname = NULL,
  @verbose             TINYINT = 0
AS

  SET NOCOUNT ON 
  SET CONCAT_NULL_YIELDS_NULL OFF

  DECLARE @now NVARCHAR(40), @err INT, @rc INT

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetEventViewerEvents: @cid=%s, @id=%d, @root_cid=%s, @root_id=%d', 0, 1, @now, @cid, @id, @root_cid, @root_id) WITH NOWAIT
  END
  
  -- GetChildren:
  -- 0: don't get child instances
  -- <> 0: get child instances

  -- Direction:
  -- 0: go backwards
  -- 1: go forwards
  IF @Direction <> 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetEventViewerEvents: only @Direction=1 is supported, caller specified @Direction=%d', 11, 1, @Direction)
    RETURN -30000
  END

  DECLARE @DisplayDBServerCTime INT
  SET @DisplayDBServerCTime = 0

  SELECT @DisplayDBServerCTime = propertyvalue
  FROM  SystemConfiguration WITH (NOLOCK) 
  WHERE groupname = '_GetEventViewerEvents' 
    AND propertyname = 'DisplayDBServerCTime' 

  IF @@rowcount = 0
    SET @DisplayDBServerCTime = 0

  -- see if the caller used the root_cid and root_id params
  IF @root_cid IS NOT NULL AND @root_id IS NOT NULL AND @root_id <> 0
    SELECT @cid = @root_cid, @id = @root_id

  -- Establish the Query's signature
  DECLARE @query_signature NVARCHAR(255)
  SET @query_signature = CONVERT(NVARCHAR(255), ISNULL(@Filters,'') + ',' + ISNULL(@OrderBy,''))
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetEventViewerEvents: @query_signature=[%s]', 0, 1, @now, @query_signature)
  END

  DECLARE @resultcmd NVARCHAR(1024)  -- first set up the result cmd to be used in returning results
  SET @resultcmd = ''
  IF @OutputTable IS NOT NULL
  BEGIN  -- if table to return to ... 
    SET @resultcmd = 'INSERT INTO ' + @OutputTable + 
    ' (obj_cid, obj_id, evt_cid, evt_id, evt_name, evt_alertstate, evt_alertstatestr, evt_priority, evt_prioritystr, ' +
      'evt_state, evt_statestr, evt_detail, evt_count, evt_seq_no, evt_datetime, note_id, resource, resourcepath, owner, _ProblemID) '
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Inserted %d row(s) into output table %s', 0, 1, @now, @rc, @OutputTable) WITH NOWAIT
    END
  END

  -- lookup the max rows setting in the SystemConfiguration table
  SELECT @MaxRows = CONVERT(INT, propertyvalue) 
  FROM SystemConfiguration WITH (NOLOCK)
  WHERE groupname = '_GetEventViewerEvents' AND propertyname = 'RETURN_LIMIT'

  -- Check if existing "recent" result set exists; if it does then use it
  DECLARE @result_set_id INT
  IF @bForceReload = 0 AND @GetChildren <> 0
  BEGIN
    DECLARE @timeout_seconds INT
    SELECT @timeout_seconds = ABS(seconds) FROM EvtViewResultCacheTimeout WITH (NOLOCK)
    SELECT @result_set_id = MAX(id)
      FROM EvtViewSignature
      WHERE obj_cid = @cid
        AND obj_id = @id
        AND last_update > DATEADD(SECOND, -1*@timeout_seconds, CURRENT_TIMESTAMP)
        AND signature = @query_signature
    IF @result_set_id IS NOT NULL
    BEGIN
      DECLARE @result_set_id_string NVARCHAR(9)
      SELECT @result_set_id_string = CAST(@result_set_id AS NVARCHAR(9))

      IF @OutputTable IS NOT NULL or @DisplayDBServerCTime = 0
      BEGIN
        EXEC ( @resultcmd + '
          SELECT obj_cid, obj_id, evt_cid, evt_id, evt_name, evt_alertstate, evt_alertstatestr, evt_priority, evt_prioritystr,
            evt_state, evt_statestr, evt_detail, evt_count, evt_seq_no, evt_datetime, note_id, resource, resourcepath, owner, _ProblemID
          FROM EvtViewResultCache
          WHERE signature_id =  ' + @result_set_id_string)
      END
      ELSE
      BEGIN
        EXEC ( 'SELECT obj_cid, obj_id, evt_cid, evt_id, evt_name, evt_alertstate, evt_alertstatestr, evt_priority, evt_prioritystr,
            evt_state, evt_statestr, evt_detail, evt_count, evt_seq_no, evt_datetime, note_id, resource, resourcepath, owner, _ProblemID, ctime
          FROM EvtViewResultCache
          WHERE signature_id =  ' + @result_set_id_string)
      END

      SET @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: _GetEventViewerEvents: Retrieved %d row(s) from EvtViewResultCache', 0, 1, @now, @rc) WITH NOWAIT
      END
      
      SELECT @numevents = event_count FROM EvtViewSignature WITH (NOLOCK) WHERE id = @result_set_id
      SET @numfilteredevents = @numevents
      RETURN 0
    END
  END
  ELSE
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: @bForceReload=%d; bypassing the previously cached result set (if any).', 0, 1, @now, @bForceReload)
    END
  END

  -- temp tables
  CREATE TABLE #obj
  (
    cid     ClassID,
    id      ObjID,
    resource    ObjName NULL,
    resourcepath  NVARCHAR(255) NULL,
    idpath              NVARCHAR(400) NULL
  )
  CREATE TABLE #obj_event
  (
    cid ClassID NOT NULL PRIMARY KEY (cid, id),
    id  ObjID NOT NULL,
    i   INT NOT NULL IDENTITY(1,1)
  )
  CREATE INDEX cidid#objevent ON #obj_event (cid,id)


  CREATE TABLE #src_obj
  (
    cid      ClassID,
    id       ObjID,
    phys_cid ClassID NULL,
    phys_id  ObjID DEFAULT 0,
    src_cid  ClassID,
    src_id   ObjID,
    nonleaf  TINYINT NOT NULL DEFAULT 0
  )
  CREATE TABLE #src_idpath
  (
    idpath  NVARCHAR(400) NOT NULL,
    cid     ClassID       NULL,
    id      ObjID         NOT NULL DEFAULT 0,
    nonleaf TINYINT       NOT NULL DEFAULT 1,
    source  TINYINT       NOT NULL DEFAULT 0
  )
  CREATE TABLE #events
  (
    obj_cid            ClassID,
    obj_id             ObjID,
    evt_cid            ClassID,
    evt_id             ObjID,
    evt_name           NVARCHAR(255) NULL,
    evt_alertstate     INT,
    evt_alertstatestr  NVARCHAR(40)  NULL,
    evt_priority       INT,
    evt_prioritystr    NVARCHAR(40)  NULL,
    evt_state          INT,
    evt_statestr       NVARCHAR(40)  NULL,
    evt_detail         NVARCHAR(255) NULL,
    evt_count          INT,
    evt_seq_no         NVARCHAR(50),
    evt_datetime       DATETIME,
    note_id            ObjID DEFAULT 0,
    resource           ObjName NULL,
    resourcepath       NVARCHAR(255) NULL,
    owner              NVARCHAR(255) NULL,
    _ProblemID         NVARCHAR(510) NULL,
    _flags             INT,
    ctime              DATETIME      NULL
  )
  CREATE TABLE #final_events
  (
    id                 INT IDENTITY(1,1) PRIMARY KEY,
    obj_cid            ClassID,
    obj_id             ObjID,
    evt_cid            ClassID,
    evt_id             ObjID,
    evt_name           NVARCHAR(255) NULL,
    evt_alertstate     INT,
    evt_alertstatestr  NVARCHAR(40)  NULL,
    evt_priority       INT,
    evt_prioritystr    NVARCHAR(40)  NULL,
    evt_state          INT,
    evt_statestr       NVARCHAR(40)  NULL,
    evt_detail         NVARCHAR(255) NULL,
    evt_count          INT,
    evt_seq_no         NVARCHAR(50),
    evt_datetime       DATETIME,
    note_id            ObjID DEFAULT 0,
    resource           ObjName NULL,
    resourcepath       NVARCHAR(255) NULL,
    owner              NVARCHAR(255) NULL,
    _ProblemID         NVARCHAR(510) NULL,
    ctime              DATETIME      NULL
  )
  CREATE TABLE #numevents
  (
    mesg_count  INT,
    excp_count  INT
  )

  -- populate the #numevents table
  INSERT INTO #numevents SELECT 0, 0

  -- declare variables
  DECLARE @id_str NVARCHAR(9), @in_maint TINYINT, @sql NVARCHAR(2000)
  SET @cid = RTRIM(@cid)
  SET @id_str = CAST(@id AS NVARCHAR(9))

  -- get the starting place
  DECLARE @MaxRowsStr NVARCHAR(9), @DirectionStr NVARCHAR(9), @mesg_id_str NVARCHAR(9), @excp_id_str NVARCHAR(9)
  IF @excp_id IS NULL
    SET @excp_id = 0
  IF @mesg_id IS NULL
    SET @mesg_id = 0
  SET @MaxRowsStr = CAST(@MaxRows AS NVARCHAR(9))
  SET @DirectionStr = CAST(@Direction AS NVARCHAR(9))
  SET @mesg_id_str = CAST(@mesg_id AS NVARCHAR(9))
  SET @excp_id_str = CAST(@excp_id AS NVARCHAR(9))

  -- Determine if the resource is in maintenance.
  -- If the resource is a physical object, just check the MaintTrueState table.
  -- If the resource is an LOB, check to see if it is linked to a physical object that is in the MaintTrueState table.
  -- This will be used later to determine which events to retrieve.  If the starting resource is in maintenance mode, then
  --   we will get all events for all resources.  If the starting resource is not in maintenance mode, then we will only
  --   get events for resources that are also not in maintenance mode.
  IF (@cid = 'LOB'
    AND EXISTS (
      SELECT 1
      FROM MaintTrueState AS m WITH (NOLOCK)
        JOIN link AS l ON l.dst_cid = m.cid AND l.dst_id = m.id
      WHERE l.src_cid = @cid AND l.src_id = @id AND l.deleted = 0 AND l.link_type = 'LOBL'
        AND m.enabled = 1
    )) OR EXISTS (
      SELECT *
      FROM MaintTrueState WITH (NOLOCK)
      WHERE cid = @cid AND id = @id AND enabled = 1
    )
    SET @in_maint = 1
  ELSE
    SET @in_maint = 0

  -- set up the filters
  DECLARE @MESGFilter NVARCHAR(1000), @EXCPFilter NVARCHAR(1000), @ObjFilter NVARCHAR(1000), @CIDFilter NVARCHAR(1000), @do_mesg TINYINT, @do_excp TINYINT
  SELECT @do_mesg = 1, @do_excp = 1
  EXEC asisp_processEVFilters @Filters, @MESGFilter OUTPUT, @EXCPFilter OUTPUT, @ObjFilter OUTPUT, @do_mesg OUTPUT, @do_excp OUTPUT, @CIDFilter OUTPUT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetEventViewerEvents: @MESGFilter=[%s]', 0, 1, @now, @MESGFilter) WITH NOWAIT
    RAISERROR('%s: _GetEventViewerEvents: @EXCPFilter=[%s]', 0, 1, @now, @EXCPFilter) WITH NOWAIT
    RAISERROR('%s: _GetEventViewerEvents: @ObjFilter=[%s]', 0, 1, @now, @ObjFilter) WITH NOWAIT
    RAISERROR('%s: _GetEventViewerEvents: @CIDFilter=[%s]', 0, 1, @now, @CIDFilter) WITH NOWAIT
    RAISERROR('%s: _GetEventViewerEvents: @do_mesg=[%d]', 0, 1, @now, @do_mesg) WITH NOWAIT
    RAISERROR('%s: _GetEventViewerEvents: @do_excp=[%d]', 0, 1, @now, @do_excp) WITH NOWAIT
  END

  DECLARE @join_clause_obj NVARCHAR(4000), @where_clause_obj NVARCHAR(4000), @where_clause_obj2 NVARCHAR(4000), @join_clause_lob NVARCHAR(4000)
  -- set up the join clause
  SET @join_clause_obj = ' JOIN #src_idpath AS o2 WITH(NOLOCK) ON o.idpath LIKE o2.idpath + ''/%'' AND o2.nonleaf = 0 '
  SET @join_clause_lob = ' JOIN #src_idpath AS o2 ON o.idpath = o2.idpath '
  IF @cid NOT IN ('ROOT', 'LOB', 'LOBC')
  BEGIN
    INSERT INTO #src_idpath (cid, id, idpath, nonleaf)
    SELECT DISTINCT oc.cid, oc.id, oc.idpath, 0
      FROM ObjPathCache AS oc WITH (NOLOCK)
      WHERE cid = @cid AND id = @id
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Inserted %d row(s) into #src_idpath for cid %s, id %d', 0, 1, @now, @rc, @cid, @id) WITH NOWAIT
    END
  END

  DECLARE @source_count INT
  SET @source_count = 0
  -- special processing for LOBs
  IF @GetChildren <> 0 AND @cid IN ('LOB', 'LOBC')
  BEGIN
    EXEC asisp_getStartingPaths @cid, @id, '#src_idpath', @verbose = @verbose
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Returned from call to asisp_getStartingPaths', 0, 1, @now) WITH NOWAIT
    END

    -- Add rows for source resources of shortcut lobs
    INSERT INTO #src_idpath 
      SELECT o.idpath, 'LOB', l.phy_id, t.nonleaf, 1
      FROM #src_idpath AS t
      JOIN lob_link AS l WITH (NOLOCK) ON l.src_id = 0 AND l.dst_id = t.id AND l.phy_cid = 'LOB'
      JOIN ObjPathCache AS o WITH (NOLOCK) ON o.cid = 'LOB' AND o.id = l.phy_id
      LEFT OUTER JOIN #src_idpath AS d ON d.cid = 'LOB' AND d.id = l.phy_id
      WHERE t.cid = 'LOB' AND d.idpath IS NULL 
    SET @source_count = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Inserted %d rows for source LOB folders for shortcuts', 0, 1, 
                 @now, @source_count) WITH NOWAIT
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Finished special LOB processing', 0, 1, @now) WITH NOWAIT
    END
  END

  -- add idpath of children that are base cids eg D2DG, so we include the children of the base objects too
  IF @GetChildren <> 0  
  BEGIN
    INSERT INTO #src_idpath (cid, id, idpath, nonleaf)
      SELECT DISTINCT RTRIM(o.cid), o.id, RTRIM(o.cid)+':'+CAST(o.id AS NVARCHAR), 0
        FROM ObjPathCacheBase AS b  (NOLOCK)
        JOIN ObjPathCache     AS o  (NOLOCK) ON o.cid = b.cid 
        JOIN #src_idpath      AS o2 (NOLOCK) ON o.idpath LIKE o2.idpath+'/%' AND o2.nonleaf = 0  -- base cid is a child of a search object
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Inserted %d row(s) into #src_idpath for children that are base cids.', 0, 1, @now, @rc) WITH NOWAIT
    END
  END

  -- get a list of objects that have messages and exceptions
  IF @GetChildren <> 0
    AND EXISTS(SELECT 1 FROM #src_idpath 
                 WHERE cid IN ('BUSC', 'ENT', 'COMP', 'MACH', 'LPAR', 'OS', 'LOB', 'LOBC', 'IPNW','IMSS','IMTM','IMDB','D2DG'))
  BEGIN
    -- Place a list of cid/id combinations in a temp table 
    -- that represent those resources that have active messages and/or exceptions
    -- 
    -- Note(1): This block of code is only an optimization for those classes that have
    --   a very large number of child objects (e.g. ENT, COMP, ...).  For those objects
    --   it is faster to use this query to reduce the # of objects joined with in the
    --   ObjPathCache.  For classes lower down in the heirarchy, there is usually none
    --   or even a negative benefit to running this portion of the query as it 
    --   adds a fixed amount of time to all queries.
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Getting a list of objects that have messages and exceptions', 0, 1, @now) WITH NOWAIT
    END
    DECLARE @obj_event_where_mesg NVARCHAR(255), @obj_event_where_excp NVARCHAR(255)
    
    IF @CIDFilter IS NOT NULL
    BEGIN
      SET @obj_event_where_mesg = ' AND _ParentCID '+@CIDFilter+' '
      SET @obj_event_where_excp = ' WHERE _ParentCID '+@CIDFilter+' '
    END 
    IF @MESGFilter IS NOT NULL
    BEGIN
      IF @obj_event_where_mesg IS NULL
        SET @obj_event_where_mesg = ' AND ' + @MESGFilter + ' '
      ELSE
        SET @obj_event_where_mesg = @obj_event_where_mesg + ' AND ' + @MESGFilter + ' '
      IF @obj_event_where_excp IS NULL
        SET @obj_event_where_excp = ' WHERE '
      ELSE
        SET @obj_event_where_excp = @obj_event_where_excp + ' AND '
      SET @obj_event_where_excp = @obj_event_where_excp + @MESGFilter + ' '
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: obj_event_where_excp=[%s], obj_event_where_mesg=[%s].', 0, 1, @now, @obj_event_where_excp, @obj_event_where_mesg) WITH NOWAIT
    END
    -- Note(1) The total cost of these two inserts is 1.6852 if no cid is added to 
    --   the WHERE clause.  The total cost is 1.0982 if there is a cid specified.
    --
    -- Note(2): Don't test for Message_C.deleted=0 as Messages are physically deleted from
    --   the table by a scheduled task.  At this time, there is no reason to check for 
    --   the flag (join to Message_C) as it reduces query performance (from 1.350 to 3.290) 
    --   and is unecessary
    --
    -- Note(3): Don't test for Exception_C.deleted=0 as exceptions are marked as deleted
    --   when the exception is 'Owned' and 'Open' as well as when they are really 'deleted'.
    --   We do not want to filter out exceptions that match this condition (Owned/Open).  Instead of 
    --   joining to the OwnedEvent and Note_A tables here, we just use this result set as a rough
    --   estimate and let the query that actually retrieves the exceptions be more granular.
    EXEC(
      'INSERT INTO #obj_event ' +
        'SELECT DISTINCT _ParentCID, _ParentID ' +
        'FROM Exception_V AS events WITH (NOLOCK) ' +
        @obj_event_where_excp
    )
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Retrieved a list of %d object(s) that have exceptions', 0, 1, @now, @rc) WITH NOWAIT
    END
    SET @sql = 
      'INSERT INTO #obj_event ' +
        'SELECT DISTINCT _ParentCID, _ParentID ' +
        'FROM Message_A AS events WITH (NOLOCK) ' +
        'LEFT OUTER JOIN #obj_event AS t ON t.cid = _ParentCID AND t.id = _ParentID ' +
        'WHERE t.cid IS NULL ' +
        @obj_event_where_mesg
    EXEC(@sql)
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Query failed. Error=%d. Query=[%s]', 0, 1, @now, @err, @sql) WITH NOWAIT
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Retrieved a list of %d object(s) that have messages', 0, 1, @now, @rc) WITH NOWAIT
    END

    SET @join_clause_obj = @join_clause_obj + 
      ' JOIN #obj_event AS oe WITH(NOLOCK) ON oe.cid = o.cid AND oe.id = o.id '
    SET @join_clause_lob = @join_clause_lob + 
      ' JOIN #obj_event AS oe WITH(NOLOCK) ON oe.cid = o.cid AND oe.id = o.id '
  END 

  -- apply the object filter
  DECLARE @ObjFilter_2 NVARCHAR(1000)
  -- set up ObjFilter_2
  SET @ObjFilter_2 = @ObjFilter
  IF @where_clause_obj2 IS NOT NULL AND @ObjFilter_2 IS NOT NULL
    SET @ObjFilter_2 = @ObjFilter_2 + ' AND (' + @where_clause_obj2 + ') '
  ELSE IF @where_clause_obj2 IS NOT NULL
    SET @ObjFilter_2 = ' (' + @where_clause_obj2 + ') '
  -- set up ObjFilter
  IF @where_clause_obj IS NOT NULL AND @ObjFilter IS NOT NULL
    SET @ObjFilter = @ObjFilter + ' AND (' + @where_clause_obj + ')'
  ELSE IF @where_clause_obj IS NOT NULL
    SET @ObjFilter = ' (' + @where_clause_obj + ')'
  -- finalize the filter variables
  IF @ObjFilter IS NULL SET @ObjFilter = ''
  ELSE SET @ObjFilter = 'WHERE (' + @ObjFilter + ')'
  IF @ObjFilter_2 IS NULL SET @ObjFilter_2 = ''
  ELSE SET @ObjFilter_2 = 'WHERE (' + @ObjFilter_2 + ')'
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetEventViewerEvents: Starting to get objects', 0, 1, @now) WITH NOWAIT
    RAISERROR('%s: _GetEventViewerEvents: @join_clause_obj=[%s]', 0, 1, @now, @join_clause_obj) WITH NOWAIT
    RAISERROR('%s: _GetEventViewerEvents: @ObjFilter=[%s]', 0, 1, @now, @ObjFilter) WITH NOWAIT
    RAISERROR('%s: _GetEventViewerEvents: @join_clause_lob=[%s]', 0, 1, @now, @join_clause_lob) WITH NOWAIT
    RAISERROR('%s: _GetEventViewerEvents: @ObjFilter_2=[%s]', 0, 1, @now, @ObjFilter_2) WITH NOWAIT
  END

  DECLARE @topLoop NVARCHAR(1000)
  DECLARE @botLoop NVARCHAR(1000)
  SET @topLoop = ' '
  SET @botLoop = ' '

  IF @GetChildren <> 0
  BEGIN
    -- get the objects in two steps
    IF CHARINDEX('#obj_event ', @join_clause_obj) > 0 
    BEGIN 
      -- pull the loop value out of SystemConfiguration
      DECLARE @objsevtsloop NVARCHAR(9) 
      SET @objsevtsloop = '10000'
      SELECT @objsevtsloop = ISNULL(propertyvalue,'10000') FROM SystemConfiguration WITH (NOLOCK) WHERE groupname='_GetEventViewerEvents' AND propertyname='ObjsEventsLoop'

      DECLARE @max_i INT, @loop_i INT
      SELECT @max_i = MAX(i)
      FROM #obj_event
      SET @loop_i = CAST(@objsevtsloop AS INT)

      -- Don't do the loop processing if the number of entries in #obj_event is less than
      -- the loop max.
      IF @loop_i < @max_i
      BEGIN
        SET @topLoop = 'DECLARE @maxi INT, @begin INT, @end INT
          SELECT @maxi= max(i) FROM #obj_event
          SET @begin=0
          WHILE @begin < @maxi
          BEGIN
          '
        SET @botLoop = 'SET @begin=@begin+' + @objsevtsloop + '   
            END
          '
        -- this next step requires the #obj_event table to be the last part of the join
        SET @join_clause_obj = @join_clause_obj + ' AND i > @begin AND i < @begin+1+'  + @objsevtsloop + ' '
      END
    END 

    EXEC(
      'INSERT INTO #obj (cid, id) ' +
      'SELECT o.cid, o.id ' +
      'FROM ObjPathCache AS o WITH (NOLOCK) ' +
      @join_clause_lob + ' ' +
      @ObjFilter + ' ' +
      'OPTION(MAXDOP 1) '
    )
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Got list of resources, now getting child resources...', 0, 1, @now) WITH NOWAIT
    END

    IF @cid = 'IPNW'
    BEGIN
      EXEC("
        INSERT INTO #obj (cid, id, idpath)
        SELECT o.cid, o.id, o.idpath
        FROM ObjPathCache AS o WITH (NOLOCK)
          JOIN #obj_event AS oe ON oe.cid = o.cid AND oe.id = o.id
        " + @ObjFilter_2 + "

        DELETE o
        FROM #obj AS o
          LEFT OUTER JOIN ObjPathCache AS o2 WITH (NOLOCK) ON o.idpath LIKE o2.idpath + '/%' AND o2.cid = '" + @cid + "' AND o2.id = " + @id_str + "
        WHERE o2.cid IS NULL
        OPTION(MAXDOP 1)
      ")
    END
    ELSE
    BEGIN
      EXEC("
        " + @topLoop + "   
        INSERT INTO #obj (cid, id)
        SELECT o.cid, o.id
        FROM ObjPathCache AS o WITH (NOLOCK)
        " + @join_clause_obj + "
        " + @ObjFilter_2 + "
        OPTION(MAXDOP 1)  " + "
        " + @botLoop + "
      ")
    END
    SET @rc = @rc + @@ROWCOUNT
  END
  ELSE
  BEGIN
    IF @cid = 'LOB'
    BEGIN
      -- First insert the LOB we started with
      INSERT INTO #obj (cid, id, resource, resourcepath)
      SELECT o.cid, o.id, o.resource,
      CASE                  
          WHEN CHARINDEX('LOBC/', o.resourcepath) = 1 THEN 
            SUBSTRING(o.resourcepath, LEN('LOBC/')+1, LEN(o.resourcepath) - LEN('LOBC/'))
          ELSE o.resourcepath 
        END        
      FROM ObjPathCache AS o WITH (NOLOCK)
      WHERE o.cid = 'LOB' AND o.id = @id
      OPTION (MAXDOP 1)
      INSERT INTO #obj (cid, id, resource, resourcepath)
      SELECT o.cid, o.id, o.resource,
      CASE                  
          WHEN CHARINDEX('LOBC/', o.resourcepath) = 1 THEN 
            SUBSTRING(o.resourcepath, LEN('LOBC/')+1, LEN(o.resourcepath) - LEN('LOBC/'))
          ELSE o.resourcepath 
        END        
      FROM ObjPathCache AS o WITH (NOLOCK)
        JOIN link AS l WITH (NOLOCK) ON l.dst_cid = o.cid AND l.dst_id = o.id AND l.deleted = 0 AND l.link_type = 'LOBL'
      WHERE l.src_cid = @cid AND l.src_id = @id
      OPTION (MAXDOP 1)
    END
    ELSE
      INSERT INTO #obj (cid, id, resource, resourcepath)
      SELECT o.cid, o.id, o.resource,
        CASE                  
          WHEN CHARINDEX('BUSC/', o.resourcepath) = 1 THEN      
            SUBSTRING(o.resourcepath, LEN('BUSC/')+1, LEN(o.resourcepath) - LEN('BUSC/'))
          ELSE o.resourcepath 
        END        
      FROM ObjPathCache AS o WITH (NOLOCK)
      WHERE o.cid = @cid AND o.id = @id
      OPTION (MAXDOP 1)
    SET @rc = @@ROWCOUNT
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetEventViewerEvents: Finished getting %d objects', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- create index
  CREATE CLUSTERED INDEX i#obj_cid_id ON #obj(cid, id)
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetEventViewerEvents: Finished creating CLUSTERED INDEX ON #obj', 0, 1, @now) WITH NOWAIT
  END

  -- set up the order by clause
  DECLARE @orderby_clause NVARCHAR(1000), @orderby_clause_evt NVARCHAR(1000), @orderby_clause_excp NVARCHAR(1000), @orderby_clause_mesg NVARCHAR(1000)

  IF @OutputTable IS NOT NULL or @DisplayDBServerCTime = 0
  BEGIN
    IF @OrderBy IS NOT NULL AND CHARINDEX('evt_datetime', @OrderBy) = 0
      SET @orderby_clause_evt = 'ORDER BY ' + @OrderBy + ', evt_datetime DESC'
    ELSE IF @OrderBy IS NULL
      SET @orderby_clause_evt = 'ORDER BY evt_datetime DESC'
    ELSE
      SET @orderby_clause_evt = 'ORDER BY ' + @OrderBy
  END
  ELSE
  BEGIN
    IF @OrderBy IS NOT NULL AND CHARINDEX('ctime', @OrderBy) = 0
      SET @orderby_clause_evt = 'ORDER BY ' + @OrderBy + ', ctime DESC'
    ELSE IF @OrderBy IS NULL
      SET @orderby_clause_evt = 'ORDER BY ctime DESC'
    ELSE
      SET @orderby_clause_evt = 'ORDER BY ' + @OrderBy
  END

  SET @orderby_clause = REPLACE(@orderby_clause_evt, 'evt_datetime', '_DATE')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_alertstatestr', 'ev1.label')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_alertstate', 'events._AlertState')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_prioritystr', 'ev2.label')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_priority', '_Priority')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_seq_no', '_SEQ_NO')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_name', 'events.name')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_count', '_nEXCH')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_statestr', 'md._State')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_state', '_StateID')
  SET @orderby_clause = REPLACE(@orderby_clause, 'note_id', 'noteid')
  SET @orderby_clause = REPLACE(@orderby_clause, 'evt_id', 'events.id')
  SET @orderby_clause = REPLACE(@orderby_clause, 'obj_id', '_ParentID')
  SET @orderby_clause = REPLACE(@orderby_clause, 'owner', 'note._UserID')
  SET @orderby_clause_evt = REPLACE(@orderby_clause_evt, 'resource', 'e.resource')
  -- @OrderBy may include _ProblemID, but column name matches

  -- pull out evt_cid if specified
  IF CHARINDEX('evt_cid', @orderby_clause) > 0
  BEGIN
    DECLARE @evt_cid_index INT, @temp_index INT, @temp_clause NVARCHAR(1000)
    SET @evt_cid_index = CHARINDEX(',', @orderby_clause, CHARINDEX('evt_cid', @orderby_clause))
    -- if the index is 0, this means that it is probably the last item on the list, so find the last comma
    IF @evt_cid_index = 0
    BEGIN
      SET @evt_cid_index = CHARINDEX(',', @orderby_clause, 1)
      WHILE @evt_cid_index > 0
      BEGIN
        SET @temp_index = @evt_cid_index
        SET @evt_cid_index = CHARINDEX(',', @orderby_clause, @evt_cid_index + 1)
      END
      IF @temp_index IS NOT NULL AND @temp_index > 0
        SET @orderby_clause = SUBSTRING(@orderby_clause, 1, @temp_index - 1)
    END
    ELSE
    BEGIN
      SET @temp_clause = SUBSTRING(@orderby_clause, 1, CHARINDEX('evt_cid', @orderby_clause) - 1)
      SET @orderby_clause = @temp_clause + SUBSTRING(@orderby_clause, @evt_cid_index + 1, LEN(@orderby_clause) - @evt_cid_index)
    END
  END
  -- object-specific order by clauses
  SET @orderby_clause_mesg = @orderby_clause
  SET @orderby_clause_excp = @orderby_clause
  SET @orderby_clause_excp = REPLACE(@orderby_clause_excp, 'evt_detail', '_EXCP_CD')
  SET @orderby_clause_mesg = REPLACE(@orderby_clause_mesg, 'evt_detail', '_REASON_CD')

  -- set up the message filter
  DECLARE @where_clause_mesg NVARCHAR(1000)
  SET @where_clause_mesg = N' WHERE ((oe.eventid IS NULL OR oe.eventid = 0) OR (oe.eventid IS NOT NULL AND note._Status = ''Open''))'
  IF @MESGFilter IS NOT NULL
    SET @where_clause_mesg = @where_clause_mesg + N' AND (' + @MESGFilter + N')'
  IF @do_mesg = 1 AND CHARINDEX('_nEXCH', @orderby_clause_mesg) > 0
    SET @do_mesg = 0

  -- remove any objects that are in maintenance, if necessary
  IF @in_maint = 0
  BEGIN
    DELETE o
    FROM #obj AS o
      JOIN MaintTrueState AS m WITH (NOLOCK) ON m.cid = o.cid AND m.id = o.id
    WHERE m.enabled = 1
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents:  Removed %d resources that are in-maintenance.', 0, 1, @now, @rc) WITH NOWAIT
    END
  END

  -- Populate the table with messages
  IF @do_mesg = 1
  BEGIN
    CREATE TABLE #mesg_ids
    (
      cid      ClassID NOT NULL,
      id       ObjID NOT NULL,
      mesg_id  INT NOT NULL
    )

    -- This clause selects the id of the most relevant message for each object into a temp table
    -- This is done because the messages that come before the one selected are no longer relevant to the object
    INSERT INTO #mesg_ids
    SELECT DISTINCT d.cid, d.id, m.id
    FROM Message_A AS m WITH (NOLOCK)
      JOIN #obj AS d ON m._ParentCID = d.cid AND m._ParentID = d.id
    WHERE m._AlertState > 1 AND m._Priority < 5
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Found %d messages with AlertState>1', 0, 1, @now, @rc) WITH NOWAIT
    END

    INSERT INTO #mesg_ids
    SELECT DISTINCT d.cid, d.id, m.id
    FROM Message_A AS m WITH (NOLOCK)
      JOIN #obj AS d ON m._ParentCID = d.cid AND m._ParentID = d.id
    WHERE m._MESSAGE <> 'Ownership' AND m._Priority < 5
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Found %d non-ownership messages', 0, 1, @now, @rc) WITH NOWAIT
    END

    CREATE TABLE #ids
    (
      id   INT NOT NULL PRIMARY KEY
    )

    INSERT INTO #ids SELECT DISTINCT MAX(mesg_id) FROM #mesg_ids GROUP BY cid, id
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Found %d distinct message id''s', 0, 1, @now, @rc) WITH NOWAIT
      RAISERROR('%s: _GetEventViewerEvents: Starting to get messages', 0, 1, @now) WITH NOWAIT
      RAISERROR('%s: _GetEventViewerEvents: @where_clause_mesg=[%s]', 0, 1, @now, @where_clause_mesg) WITH NOWAIT
      RAISERROR('%s: _GetEventViewerEvents: @orderby_clause_mesg=[%s]', 0, 1, @now, @orderby_clause_mesg) WITH NOWAIT
    END
    
    EXEC("
    INSERT INTO #events(obj_cid, obj_id, evt_cid, evt_id, evt_name, evt_alertstate, evt_alertstatestr, evt_priority, evt_prioritystr, 
        evt_state, evt_statestr, evt_detail, evt_count, evt_seq_no, evt_datetime, note_id, resource, resourcepath, owner, _ProblemID, _flags, ctime)
      SELECT DISTINCT events._ParentCID, events._ParentID, N'MESG', events.id, events._MESSAGE AS name, ISNULL(events._AlertState, 0), NULL, ISNULL(events._Priority, 0), NULL, 
          ISNULL(events._StateID, 0), md._State, events._REASON_CD, 0, ISNULL(events._SEQ_NO,''), _DATE, ISNULL(oe.noteid, 0), d.resource, d.resourcepath, note._UserID, ISNULL(note._ProblemID, ''), 
          events._flags, events.ctime
        FROM #ids AS ids
          JOIN Message_V AS events WITH (NOLOCK) ON ids.id = events.id
          JOIN #obj AS d ON events._ParentCID = d.cid AND events._ParentID = d.id
          LEFT OUTER JOIN OwnedEvent AS oe WITH (NOLOCK) ON events.id = oe.eventid AND oe.eventcid = 'MESG'
          LEFT OUTER JOIN Note_A AS note WITH (NOLOCK) ON oe.noteid = note.id
          LEFT OUTER JOIN StateType_A_LOCAL AS md WITH (NOLOCK) ON md.id = events._StateID AND md.langid = @@LANGID
      " + @where_clause_mesg + "
    ")
    SET @rc = @@ROWCOUNT    
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Finished getting %d message(s)', 0, 1, @now, @rc) WITH NOWAIT
    END

    UPDATE #numevents SET mesg_count = @rc
    DROP TABLE #ids
  END

  -- set up the exception filters
  DECLARE @where_clause_excp NVARCHAR(1000)
  SET @where_clause_excp = N' WHERE (events.deleted = 0 OR (oe.noteid <> 0 AND oe.noteid IS NOT NULL AND note._Status = ''Open''))'
  IF @EXCPFilter IS NOT NULL
    SET @where_clause_excp = @where_clause_excp + N' AND (' + @EXCPFilter + N')'
  IF @do_excp = 1 AND (CHARINDEX('md._State', @orderby_clause_excp) > 0 OR CHARINDEX('_StateID', @orderby_clause_excp) > 0)
    SET @do_excp = 0

  -- Populate the table with exceptions
  IF @do_excp = 1
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: @where_clause_excp=[%s]', 0, 1, @now, @where_clause_excp) WITH NOWAIT
      RAISERROR('%s: _GetEventViewerEvents: @orderby_clause_excp=[%s]', 0, 1, @now, @orderby_clause_excp) WITH NOWAIT
      RAISERROR('%s: _GetEventViewerEvents: Starting to get exceptions', 0, 1, @now) WITH NOWAIT
    END

    EXEC("
    INSERT INTO #events(obj_cid, obj_id, evt_cid, evt_id, evt_name, evt_alertstate, evt_alertstatestr, evt_priority, evt_prioritystr, 
        evt_state, evt_statestr, evt_detail, evt_count, evt_seq_no, evt_datetime, note_id, resource, resourcepath, owner, _ProblemID, _flags, ctime)
      SELECT DISTINCT _ParentCID, _ParentID, N'EXCP', events.id, events._EXCP_NAME AS name, ISNULL(events._AlertState, 0), NULL, ISNULL(_Priority, 0), NULL,
        -1, NULL, _EXCP_CD, _nEXCH, _SEQ_NO, _DATE, ISNULL(noteid, 0), d.resource, d.resourcepath, note._UserID, ISNULL(note._ProblemID, ''), _flags, events.ctime
      FROM Exception_V AS events WITH (NOLOCK)
        JOIN #obj AS d ON events._ParentCID = d.cid AND events._ParentID = d.id
        LEFT OUTER JOIN OwnedEvent AS oe WITH (NOLOCK) ON oe.eventid = events.id AND oe.eventcid = 'EXCP'
        LEFT OUTER JOIN Note_A AS note WITH (NOLOCK) ON oe.noteid = note.id
      " + @where_clause_excp + "
    ")
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Finished getting %d exception(s)', 0, 1, @now, @rc) WITH NOWAIT
    END
    UPDATE #numevents SET excp_count = @rc
  END

  -- Remove "LOB only" events as needed
  IF @cid = 'LOB'
  BEGIN
    SET @rc = 0
    
    -- If @GetChildren = 1, then check to see if any "source" resources were added to #src_idpath
    IF @GetChildren <> 0
    BEGIN
      IF @source_count > 0 
      BEGIN
        -- Since source resources were also searched, delete the "LOB only" events
        DELETE e 
          FROM #events AS e
          JOIN #src_idpath AS s ON e.obj_cid = s.cid and e.obj_id = s.id
          WHERE e.obj_cid = 'LOB' AND s.source = 1 AND ( e._flags & 16 = 16 )
        SET @rc = @@ROWCOUNT
      END
    END
    -- Children were not retrieved (@GetChildren = 0), so source resources, if any, were added to #obj
    ELSE
    BEGIN
      -- Delete the "LOB only" events -- the #obj will contain only the starting cid/id for a LOB folder,
      -- unless the starting cid/id is a shortcut. In this case the source cid/id was added to #obj and
      -- potentially caused "LOB only" events that should not be returned.
      DELETE FROM #events 
        WHERE obj_cid = 'LOB' AND obj_id <> @id AND ( _flags & 16 = 16 )
      SET @rc = @@ROWCOUNT
    END

    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetEventViewerEvents: Removed %d "LOB only" event(s)', 0, 1, @now, @rc) WITH NOWAIT
    END
  END

  -- Populate the final table
  EXEC("
    INSERT INTO #final_events (obj_cid, obj_id, evt_cid, evt_id, evt_name, evt_alertstate, evt_alertstatestr, evt_priority, evt_prioritystr,
        evt_state, evt_statestr, evt_detail, evt_count, evt_seq_no, evt_datetime, note_id, resource, resourcepath, owner, _ProblemID, ctime)
    SELECT e.obj_cid, e.obj_id, e.evt_cid, e.evt_id, e.evt_name, e.evt_alertstate, e.evt_alertstatestr, e.evt_priority, e.evt_prioritystr,
           e.evt_state, e.evt_statestr, e.evt_detail, e.evt_count, e.evt_seq_no, e.evt_datetime, e.note_id, o.resource,
           CASE                  
             WHEN CHARINDEX('BUSC/', o.resourcepath) = 1 THEN      
               SUBSTRING(o.resourcepath, LEN('BUSC/')+1, LEN(o.resourcepath) - LEN('BUSC/'))
             WHEN CHARINDEX('LOBC/', o.resourcepath) = 1 THEN 
               SUBSTRING(o.resourcepath, LEN('LOBC/')+1, LEN(o.resourcepath) - LEN('LOBC/'))
             ELSE o.resourcepath 
           END,        
           ISNULL(e.owner,''), e._ProblemID, e.ctime
    --Above line changed for OA13558 - set null entries to blank
    FROM #events AS e 
      JOIN ObjPathCache AS o WITH (NOLOCK) ON e.obj_cid = o.cid AND e.obj_id = o.id
    " + @orderby_clause_evt + "
  ")
  SET @numevents = @@ROWCOUNT
  SET @numfilteredevents = @numevents

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetEventViewerEvents: Finished processing final table', 0, 1, @now) WITH NOWAIT
  END

  DECLARE @WHERE_id_clause NVARCHAR(255)
  IF @MaxRows > 0
    SET @WHERE_id_clause = 'WHERE fev.id <= ' + @MaxRowsStr
  ELSE
    SET @WHERE_id_clause = ''

  -- Store the final results in the result cache
  -- EXPLICITLY NO TRANSACTION LOGIC
  IF @GetChildren <> 0
  BEGIN
    INSERT INTO EvtViewSignature(obj_cid, obj_id, last_update, signature, event_count)
      SELECT @cid, @id, CURRENT_TIMESTAMP, @query_signature, @numevents
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT, @result_set_id = SCOPE_IDENTITY()
    IF @err = 0 AND @rc = 1
    BEGIN
      DECLARE @result_set_id_str NVARCHAR(9)
      SET @result_set_id_str = CAST(@result_set_id AS NVARCHAR(9))
      EXEC(
        'INSERT INTO EvtViewResultCache(signature_id, obj_cid, obj_id, evt_cid, evt_id, evt_name, evt_alertstate, evt_alertstatestr, evt_priority, evt_prioritystr, ' +
            'evt_state, evt_statestr, evt_detail, evt_count, evt_seq_no, evt_datetime, note_id, resource, resourcepath, owner, _ProblemID, ctime) ' +
          'SELECT ' + @result_set_id_str + ', obj_cid, obj_id, evt_cid, evt_id, evt_name, evt_alertstate, evt_alertstatestr, evt_priority, evt_prioritystr, ' +
            'evt_state, evt_statestr, evt_detail, evt_count, evt_seq_no, evt_datetime, note_id, resource, resourcepath, owner, _ProblemID, ctime ' +
          'FROM #final_events AS fev ' +
          @WHERE_id_clause
      )
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: _GetEventViewerEvents: Inserted %d row(s) into EvtViewResultCache', 0, 1, @now) WITH NOWAIT
      END
    END
  END
  
  IF @err <> 0
    DELETE FROM EvtViewSignature WHERE id = @result_set_id

  -- More specific ORDER BY clause is not necessary as already inserted into #final_events in order

    IF @OutputTable IS NOT NULL or @DisplayDBServerCTime = 0
    BEGIN
      EXEC(@resultcmd + '
      SELECT obj_cid, 
             obj_id, 
             evt_cid, 
             evt_id, 
             evt_name, 
             evt_alertstate, 
             evt_alertstatestr, 
             evt_priority, 
             evt_prioritystr, 
             evt_state, 
             evt_statestr, 
             evt_detail, 
             evt_count, 
             evt_seq_no, 
             evt_datetime, 
             note_id, 
             resource, 
             resourcepath, 
             owner,
             _ProblemID
        FROM #final_events as fev
        ' + @WHERE_id_clause + '
        ORDER by fev.id'  -- More specific ORDER BY clause is not necessary as already inserted into #final_events in order
    )
  END
  ELSE
  BEGIN
      EXEC('
      SELECT obj_cid, 
             obj_id, 
             evt_cid, 
             evt_id, 
             evt_name, 
             evt_alertstate, 
             evt_alertstatestr, 
             evt_priority, 
             evt_prioritystr, 
             evt_state, 
             evt_statestr, 
             evt_detail, 
             evt_count, 
             evt_seq_no, 
             evt_datetime, 
             note_id, 
             resource, 
             resourcepath, 
             owner,
             _ProblemID,
             ctime
        FROM #final_events as fev
        ' + @WHERE_id_clause + '
        ORDER by fev.id'  -- More specific ORDER BY clause is not necessary as already inserted into #final_events in order
    )
  END

  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetEventViewerEvents: Finished. Number of events returned = %d', 0, 1, @now, @rc) WITH NOWAIT
  END
GO
