-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004,2006 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_creatediscoverylinks.sql, tbsm.sql, tbsm_base31e
-- SID: 1.8.1.9
-- Date Modified: 08/01/23 15:01:31
--
-- Module Description
--   Defines the asisp_createDiscoveryLinks stored procedure

USE Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_createDiscoveryLinks
PRINT 'Creating procedure asisp_createDiscoveryLinks'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_createDiscoveryLinks' AND type = 'P')
  DROP PROCEDURE asisp_createDiscoveryLinks
GO

-- File: archive/db/sql/procs/asisp_creatediscoverylinks.sql, tbsm.sql, tbsm_base31e
-- SID: 1.8.1.9
-- Date Modified: 08/01/23 15:01:31
-- Utility procedure used by _DiscoverXXXResources procedures to find/create links.
-- The calling procedure creates and populates a table called #sql_vars with the following columns:
/*
    orderno         INT NOT NULL,           -- order in which row is processed
    src_cid         NVARCHAR(4) NOT NULL,   -- literal cid of the source side of the links
    src_cid_col     NVARCHAR(12) NOT NULL,  -- column in workTable that contains the src_cid
                                            -- Can be a literal cid, but then must be quoted
    src_id          NVARCHAR(12) NOT NULL,  -- column in workTable that contains the src_id
    dst_cid         NVARCHAR(4) NOT NULL,   -- literal cid of the destination side of the links
    dst_cid_col     NVARCHAR(12) NOT NULL,  -- column in workTable that contains the dst_cid
                                            -- Can be a literal cid, but then must be quoted
    dst_id          NVARCHAR(12) NOT NULL,  -- column in workTable that contains the dst_id
    agg_col         NVARCHAR(12) NULL       -- if non-NULL, the name of the column in workTable
                                            -- that contains the id of the aggregate object to
                                            -- which the destination objects should also be linked
                                          -- NOTE:  THE agg_col FIELD IS NOW OBSOLETE DUE TO SEVERE PERFORMANCE PROBLEMS RELATED TO THIS AGGR LINK
                                          --        THE CODE BELOW THAT USES THIS FIELD HAD BEEN REMOVED IN DEFECT 103669
*/
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004,2006 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- For each row in the table the procedure executes a series of templated SQL using the
-- column values as variables.
CREATE PROCEDURE asisp_createDiscoveryLinks
  @workTable  ObjName,
  @verbose    TINYINT = 0
AS
  DECLARE @now NVARCHAR(40)
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_createDiscoveryLinks @workTable=''%s'', @verbose=%d', 0, 1, @now, @workTable, @verbose) WITH NOWAIT
  END

  DECLARE @check_link INT, @newlink_rc INT
  DECLARE @err INT, @rc INT, @cid ClassID, @table NVARCHAR(255)
  DECLARE @id ObjID, @cname ObjName
  DECLARE @link_type NVARCHAR(20)
  SET @newlink_rc = 0

  -- Added Txn Logic. 
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_createDiscoveryLinks
  ELSE
    SAVE TRANSACTION asisp_createDiscoveryLinks

  -- Added cleanup logic for OA23743 to remove links where D2VW resources are linked to themselves
  IF EXISTS (SELECT 1 FROM link WHERE src_cid = dst_cid AND src_id = dst_id AND src_cid = 'D2VW' AND link_type = 'GENL')
  BEGIN
    DELETE FROM link WHERE src_cid = dst_cid AND src_id = dst_id AND src_cid = 'D2VW' AND link_type = 'GENL'
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_createDiscoveryLinks: Error deleting bad D2VW-D2VW links. Error=%d', 0, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_createDiscoveryLinks
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_createDiscoveryLinks: Deleted %d bad D2VW-D2VW link(s)', 0, 1, @now, @rc) WITH NOWAIT
    END
  END

  -- For each row in #sql_vars we drive a series of templated SQL expressions which will do the following:
  -- 1. Lookup previous src->dst links
  -- 2. Create any new src->dst links required
  DECLARE createlinkc CURSOR FOR
    SELECT orderno, src_cid, src_cid_col, src_id, dst_cid, dst_cid_col, dst_id, agg_col
    FROM #sql_vars
    ORDER BY orderno
  OPEN createlinkc
  DECLARE
    @sql          NVARCHAR(4000),
    @orderno      INT,
    @src_cid      NVARCHAR(4),
    @src_cid_col  NVARCHAR(12),
    @src_id       NVARCHAR(12),
    @dst_cid      NVARCHAR(4),
    @dst_cid_col  NVARCHAR(12),
    @dst_id       NVARCHAR(12),
    @agg_col      NVARCHAR(12)

  -- #MoveOldLinks is at the ID level, like DB2Discovery ot IMSDiscovery
  -- #sql_vars is at the CID level
  CREATE TABLE #MoveOldLinks 
  (
	old_parent_id NVARCHAR(12) null, 
	old_parent_cid NVARCHAR(4) null, 
	old_unique_parent_id NVARCHAR(12) null, 
	old_unique_parent_cid NVARCHAR(4) null, 
	new_parent_id NVARCHAR(12) null, 
	new_parent_cid NVARCHAR(4) null, 
	new_unique_parent_id NVARCHAR(12) null ,
	new_unique_parent_cid NVARCHAR(4) null ,
	new_obj_id NVARCHAR(12) null,
	new_obj_cid NVARCHAR(4) null,
	to_be_moved int null
  )

  DECLARE @flag_no_link_tables TINYINT, @flag_containment TINYINT

  FETCH NEXT FROM createlinkc INTO @orderno, @src_cid, @src_cid_col, @src_id, @dst_cid, @dst_cid_col, @dst_id, @agg_col
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    TRUNCATE TABLE #MoveOldLinks

    SET @link_type = CASE WHEN @src_cid = 'D2VW' AND @dst_cid IN ('D2VW','D2TB') THEN 'GENL'
                          WHEN @src_cid = 'IMPG' AND @dst_cid = 'IMDA'           THEN 'GENL'
                          ELSE    'ASHL' END

    SELECT @flag_no_link_tables = flag_no_link_tables, @flag_containment = flag_containment
    FROM link_type
    WHERE link_type = @link_type

    SET @check_link = 1
    SET @sql = "
      IF NOT EXISTS(SELECT 1 FROM "+@workTable+"
                      WHERE "+@src_cid_col+" = '"+@src_cid+"' AND "+@src_id+" IS NOT NULL
                        AND "+@dst_cid_col+" = '"+@dst_cid+"' AND "+@dst_id+" IS NOT NULL )
        SET @check_link = 0
      "
    EXEC sp_executesql @sql, N'@check_link INT OUTPUT', @check_link = @check_link OUTPUT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      IF @check_link = 0
        RAISERROR('%s: asisp_createDiscoveryLinks: no entries in %s for src(%s %s %s) dst( %s %s %s)'
                   , 0, 1, @now, @workTable, @src_cid_col, @src_cid, @src_id, @dst_cid_col, @dst_cid, @dst_id) WITH NOWAIT
      ELSE
        RAISERROR('%s: asisp_createDiscoveryLinks: Discovering links in %s for src(%s %s %s) dst( %s %s %s)'
                   , 0, 1, @now, @workTable, @src_cid_col, @src_cid, @src_id, @dst_cid_col, @dst_cid, @dst_id) WITH NOWAIT
    END

    IF @check_link = 1
    BEGIN
      -- Lookup previously existing links
      SET @sql = "
      UPDATE w SET linked_flag = " + CAST(@orderno AS NVARCHAR) + "
      FROM " + @workTable + " AS w
        JOIN link AS l ON
            l.src_cid = " + @src_cid_col + "
        AND l.src_id  = " + @src_id + "
        AND l.dst_cid = " + @dst_cid_col + "
        AND l.dst_id  = " + @dst_id + "
      WHERE " + @src_cid_col + " = '" + @src_cid + "'
        AND " + @dst_cid_col + " = '" + @dst_cid + "'
        AND l.link_type = '"+@link_type+"'
        AND l.deleted = 0"
      IF @verbose >= 2
        PRINT @sql
      EXEC(@sql)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_createDiscoveryLinks: Error looking up %s->%s relationships. Error=%d', 0, 1, @now, @src_cid, @dst_cid, @err) WITH NOWAIT
        DEALLOCATE createlinkc
        -- Added Txn Logic. 
        ROLLBACK TRANSACTION asisp_createDiscoveryLinks
        RETURN -30000
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_createDiscoveryLinks:   Found %d previous %s->%s relationship(s)', 0, 1, @now, @rc, @src_cid, @dst_cid) WITH NOWAIT
      END

      -- Perform garbage collection on deleted links
      IF @link_type = 'ASHL'
        SET @sql = "
            DELETE FROM il 
              FROM " + @src_cid + "_" + @dst_cid + "_ASHL AS il
              JOIN " + @workTable + " AS w ON w." + @src_id + " = il.src_id AND w." + @dst_id + " = il.dst_id
              WHERE w.linked_flag <> " + CAST(@orderno AS NVARCHAR) + "
                AND " + @src_cid_col + " = '" + @src_cid + "'
                AND " + @dst_cid_col + " = '" + @dst_cid + "'"
      ELSE 
        SET @sql = "
            DELETE l 
              FROM "+@workTable+" AS w
              JOIN link AS l ON l.src_cid = " + @src_cid_col + "
                            AND l.src_id  = " + @src_id + "
                            AND l.dst_cid = " + @dst_cid_col + "
                            AND l.dst_id  = " + @dst_id + "
              WHERE " + @src_cid_col + " = '" + @src_cid + "'
                AND " + @dst_cid_col + " = '" + @dst_cid + "'
                AND l.link_type = '"+@link_type+"'
                AND l.deleted = 1  -- previous deleted link "
      IF @verbose >= 2
        PRINT @sql
      EXEC(@sql)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_createDiscoveryLinks: Error deleting obsolete links for %s %s %s. Error=%d', 0, 1, @now, @src_cid, @dst_cid, @link_type, @err) WITH NOWAIT
        DEALLOCATE createlinkc
        -- Added Txn Logic. 
        ROLLBACK TRANSACTION asisp_createDiscoveryLinks
        RETURN -30000
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_createDiscoveryLinks:   Deleted %d old row(s) for %s %s %s', 0, 1, @now, @rc, @src_cid, @dst_cid, @link_type) WITH NOWAIT
      END

      -- First Retrieve Parent Info for DB2 Objects.
      IF ( @dst_cid in ( 'D2VW','D2DB','D2TB','D2IX','D2IP') )
      BEGIN
       SET @sql = "
         INSERT INTO #MoveOldLinks (new_parent_id,new_parent_cid,new_unique_parent_id,new_unique_parent_cid,new_obj_id,new_obj_cid,to_be_moved) 
	 SELECT DISTINCT " + @src_id + ", " + @src_cid_col + ", d2dg_id,'D2DG',obj_id,obj_cid,0
         FROM " + @workTable + "  
         WHERE " + @src_cid_col + " = '" + @src_cid + "'
         AND " + @src_id + " IS NOT NULL
         AND " + @dst_cid_col + " = '" + @dst_cid + "'
         AND " + @dst_id + " IS NOT NULL
         AND linked_flag <> " + CAST(@orderno AS NVARCHAR) 
      END
      -- more DB2 Objects.....
      ELSE IF ( @dst_cid in ( 'D2TS','D2TP') )
      BEGIN
       SET @sql = "
        INSERT INTO #MoveOldLinks (new_parent_id,new_parent_cid,new_unique_parent_id,new_unique_parent_cid,new_obj_id,new_obj_cid,to_be_moved) 
	SELECT DISTINCT " + @src_id + ", " + @src_cid_col + ", dbdb_id,dbdb_cid,obj_id,obj_cid,0
        FROM " + @workTable + "  
        WHERE " + @src_cid_col + " = '" + @src_cid + "'
        AND " + @src_id + " IS NOT NULL
        AND " + @dst_cid_col + " = '" + @dst_cid + "'
        AND " + @dst_id + " IS NOT NULL
        AND linked_flag <> " + CAST(@orderno AS NVARCHAR) 
      END  
      -- now work with IMS Objects .....
      ELSE IF ( @dst_cid in ( 'IMTX','IMAR','IMPA') )
      BEGIN
       SET @sql = "
         INSERT INTO #MoveOldLinks (new_parent_id,new_parent_cid,new_unique_parent_id,new_unique_parent_cid,new_obj_id,new_obj_cid,to_be_moved) 
	 SELECT DISTINCT " + @src_id + ", " + @src_cid_col + ", subsystem_id,imsu_cid,obj_id,obj_cid,0
         FROM " + @workTable + "  
         WHERE " + @src_cid_col + " = '" + @src_cid + "'
         AND " + @src_id + " IS NOT NULL
         AND " + @dst_cid_col + " = '" + @dst_cid + "'
         AND " + @dst_id + " IS NOT NULL
         AND linked_flag <> " + CAST(@orderno AS NVARCHAR) 
      END
      ELSE -- Just make a NULL row for all the others.
      BEGIN
       SET @sql = "
         INSERT INTO #MoveOldLinks (new_parent_id,new_parent_cid,new_obj_id,new_obj_cid,to_be_moved) 
	 SELECT DISTINCT " + @src_id + ", " + @src_cid_col + ",obj_id,obj_cid,0
         FROM " + @workTable + "  
         WHERE " + @src_cid_col + " = '" + @src_cid + "'
         AND " + @src_id + " IS NOT NULL
         AND " + @dst_cid_col + " = '" + @dst_cid + "'
         AND " + @dst_id + " IS NOT NULL
         AND linked_flag <> " + CAST(@orderno AS NVARCHAR) 
      END

      IF @verbose >= 2
        PRINT @sql
      EXEC (@sql)

      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_createDiscoveryLinks: Error inserting into #MoveOldLinks for %s %s %s. Error=%d', 0, 1, @now, @src_cid, @dst_cid, @link_type, @err) WITH NOWAIT
        DEALLOCATE createlinkc
        -- Added Txn Logic. 
        ROLLBACK TRANSACTION asisp_createDiscoveryLinks
        RETURN -30000
      END

      IF EXISTS ( select * from #MoveOldLinks )
      BEGIN
        IF ( @dst_cid in ( 'D2VW','D2DB','D2TB','D2IX','D2IP','D2TS','D2TP') )
        BEGIN
  	  -- FIND the current unique parent
          -- IF @link_type = 'ASHL'
	  SET @sql=""
	    SELECT @sql = "UPDATE p set p.old_unique_parent_id = c._UniqParentID, p.old_unique_parent_cid = c._UniqParentCID
		           from #MoveOldLinks p JOIN " + cname + "_C c on p.new_obj_id = c.id " 
		    from	  obj_class  
		    where  cid = "" + @dst_cid + ""

          IF @verbose >= 2
            PRINT @sql
	  EXEC (@sql)
	END
        ELSE IF ( @dst_cid in ( 'IMTX','IMAR','IMPA') )
        BEGIN
  	  -- FIND the current unique parent
	  SET @sql=""
	    SELECT @sql = "UPDATE p set p.old_unique_parent_id = c._SubsystemID, p.old_unique_parent_cid = c._SubsystemCID
		           from #MoveOldLinks p JOIN " + cname + "_C c on p.new_obj_id = c.id " 
		    from	  obj_class  
		    where  cid = "" + @dst_cid + ""

          IF @verbose >= 2
            PRINT @sql
	  EXEC (@sql)
	END
      END

      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_createDiscoveryLinks: Error retieving current UNIQUE parent for %s %s %s. Error=%d', 0, 1, @now, @src_cid, @dst_cid, @link_type, @err) WITH NOWAIT
        DEALLOCATE createlinkc
        -- Added Txn Logic. 
        ROLLBACK TRANSACTION asisp_createDiscoveryLinks
        RETURN -30000
      END

      -- FIND the current parent
      SET @sql=""
      IF @link_type = 'ASHL'
      BEGIN
        SET @sql = "UPDATE p set p.old_parent_id = l.src_id, p.old_parent_cid = """ + @src_cid + """
		           from #MoveOldLinks p join " + @src_cid + "_" + @dst_cid + "_ASHL l on l.dst_id = p.new_obj_id
			   and old_unique_parent_id is not null "
      END
      ELSE
      BEGIN
        SET @sql = "UPDATE p set p.old_parent_id = l.src_id, p.old_parent_cid = l.src_cid
		           from #MoveOldLinks p join link l on l.dst_cid = """ + @dst_cid + """ and l.dst_id = p.new_obj_id
			   and old_unique_parent_id is not null "
      END
      IF @verbose >= 2
        PRINT @sql
      EXEC (@sql)

      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_createDiscoveryLinks: Error retieving current parent for %s %s %s. Error=%d', 0, 1, @now, @src_cid, @dst_cid, @link_type, @err) WITH NOWAIT
        DEALLOCATE createlinkc
        -- Added Txn Logic. 
        ROLLBACK TRANSACTION asisp_createDiscoveryLinks
        RETURN -30000
      END
      
      UPDATE	#MoveOldLinks
      SET	to_be_moved = 1
      WHERE	new_unique_parent_id = old_unique_parent_id 
      AND	new_unique_parent_cid = old_unique_parent_cid
      AND	old_parent_cid = new_parent_cid 
      AND	old_parent_id <> new_parent_id
      AND	old_unique_parent_id is not null

      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_createDiscoveryLinks: Error updating to_be_moved flag for %s %s %s. Error=%d', 0, 1, @now, @src_cid, @dst_cid, @link_type, @err) WITH NOWAIT
        DEALLOCATE createlinkc
        -- Added Txn Logic. 
        ROLLBACK TRANSACTION asisp_createDiscoveryLinks
        RETURN -30000
      END

      IF @verbose >= 2
        select 'Contents of #MoveOldLinks table',* from #MoveOldLinks

      IF @verbose >= 2
      BEGIN
	set @sql = "SELECT ""Contents of the Discovery table"",* FROM " + @workTable
	EXEC (@sql)
      END

      -- Update links to move objects, if appropiate
      IF @flag_no_link_tables = 0
      BEGIN
        SET @sql = "	UPDATE l 
			SET l.src_id = p.new_parent_id 
			FROM " + @src_cid + "_" + @dst_cid + "_ASHL l
			JOIN #MoveOldLinks p on l.dst_id = p.new_obj_id
			AND l.src_id = p.old_parent_id
			WHERE to_be_moved=1 "

        IF @verbose >= 2
          PRINT @sql
        EXEC(@sql)
        SELECT @err = @@ERROR, @rc = @@ROWCOUNT
        IF @err <> 0
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: asisp_createDiscoveryLinks: Error updating links (flag_no_link_tables = 0) for %s %s %s. Error=%d', 0, 1, @now, @src_cid, @dst_cid, @link_type, @err) WITH NOWAIT
          DEALLOCATE createlinkc
          -- Added Txn Logic. 
          ROLLBACK TRANSACTION asisp_createDiscoveryLinks
          RETURN -30000
        END
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: asisp_createDiscoveryLinks:   Updated %d row(s) for %s %s %s (flag_no_link_tables = 0)', 0, 1, @now, @rc, @src_cid, @dst_cid, @link_type) WITH NOWAIT
        END

      END

      SET @sql = "	UPDATE l 
			SET l.src_id = p.new_parent_id, l.src_cid = p.new_parent_cid 
			FROM link l
			JOIN #MoveOldLinks p on l.dst_id = p.new_obj_id 
			AND l.dst_cid = p.new_obj_cid
			AND l.link_type = """ + @link_type + """ 
			AND l.src_cid = old_parent_cid
			AND l.src_id = old_parent_id
			WHERE to_be_moved=1 
                        AND NOT EXISTS (SELECT 1 FROM link 
                                        WHERE dst_id  = p.new_obj_id 
                                          AND dst_cid = p.new_obj_cid 
                                          AND src_id  = p.new_parent_id 
                                          AND src_cid = p.new_parent_cid)
                 "      

      IF @verbose >= 2
        PRINT @sql
      EXEC(@sql)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_createDiscoveryLinks: Error updating link table for %s %s %s. Error=%d', 0, 1, @now, @src_cid, @dst_cid, @link_type, @err) WITH NOWAIT
        DEALLOCATE createlinkc
        -- Added Txn Logic. 
        ROLLBACK TRANSACTION asisp_createDiscoveryLinks
        RETURN -30000
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_createDiscoveryLinks:   Updated %d row(s) in link table for %s %s %s', 0, 1, @now, @rc, @src_cid, @dst_cid, @link_type) WITH NOWAIT
      END

      -- If this is a containment link then update the containment table as well
      IF @flag_containment = 1
      BEGIN
        SET @sql = "	UPDATE c 
			SET c.src_id = p.new_parent_id, c.src_cid = p.new_parent_cid 
			FROM containment c
			JOIN #MoveOldLinks p on c.dst_id = p.new_obj_id 
			AND c.dst_cid = p.new_obj_cid
			AND c.src_cid = old_parent_cid
			AND c.src_id = old_parent_id
			WHERE to_be_moved=1 "      

        IF @verbose >= 2
          PRINT @sql
        EXEC(@sql)
        SELECT @err = @@ERROR, @rc = @@ROWCOUNT
        IF @err <> 0
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: asisp_createDiscoveryLinks: Error updating containment for %s %s %s. Error=%d', 0, 1, @now, @src_cid, @dst_cid, @link_type, @err) WITH NOWAIT
          DEALLOCATE createlinkc
          -- Added Txn Logic. 
          ROLLBACK TRANSACTION asisp_createDiscoveryLinks
          RETURN -30000
        END
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: asisp_createDiscoveryLinks:   Updated %d containment row(s) for %s %s %s', 0, 1, @now, @rc, @src_cid, @dst_cid, @link_type) WITH NOWAIT
        END
      END

      -- The following select statements has been modified to ignore the rows already moved.
      -- Create new links
      IF @link_type = 'ASHL'
        SET @sql = "
       	  INSERT INTO " + @src_cid + "_" + @dst_cid + "_ASHL(src_id, dst_id)
       	  SELECT DISTINCT w." + @src_id + ", w." + @dst_id + "
          FROM " + @workTable + " w JOIN #MoveOldLinks p ON w.obj_id = p.new_obj_id and p.new_obj_cid = '" + @dst_cid + "' and p.to_be_moved = 0
          WHERE " + @src_cid_col + " = '" + @src_cid + "'
            AND " + @src_id + " IS NOT NULL
            AND " + @dst_cid_col + " = '" + @dst_cid + "'
            AND " + @dst_id + " IS NOT NULL
            AND linked_flag <> " + CAST(@orderno AS NVARCHAR)
      ELSE
        SET @sql = "
            INSERT INTO link (src_cid, src_id, dst_cid, dst_id, link_type)
              SELECT DISTINCT '"+@src_cid+"',"+@src_id+", '"+@dst_cid+"',"+@dst_id+", '"+@link_type+"'
              FROM "+@workTable+" w JOIN #MoveOldLinks p ON w.obj_id = p.new_obj_id and p.new_obj_cid = '" + @dst_cid + "' and p.to_be_moved = 0
                WHERE " + @src_cid_col + " = '" + @src_cid + "'
                  AND " + @src_id + " IS NOT NULL
                  AND " + @dst_cid_col + " = '" + @dst_cid + "'
                  AND " + @dst_id + " IS NOT NULL
                  AND linked_flag <> " + CAST(@orderno AS NVARCHAR)

      IF @verbose >= 2
        PRINT @sql
      EXEC(@sql)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      SET @newlink_rc = @newlink_rc + @rc
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_createDiscoveryLinks: Error inserting for %s %s %s. Error=%d', 0, 1, @now, @src_cid, @dst_cid, @link_type, @err) WITH NOWAIT
        DEALLOCATE createlinkc
        -- Added Txn Logic. 
        ROLLBACK TRANSACTION asisp_createDiscoveryLinks
        RETURN -30000
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_createDiscoveryLinks:   Inserted %d row(s) for %s %s %s', 0, 1, @now, @rc, @src_cid, @dst_cid, @link_type) WITH NOWAIT
      END

    END -- IF @check_link = 1

    FETCH NEXT FROM createlinkc INTO @orderno, @src_cid, @src_cid_col, @src_id, @dst_cid, @dst_cid_col, @dst_id, @agg_col
  END
  DEALLOCATE createlinkc

  -- If any new links were created, then cleanup possible obsolete *UNKNOWN* IMS & DB2 links/objects
  IF @newlink_rc > 0 
  BEGIN
    EXEC asisp_cleanupUnknownParents @workTable=@workTable, @verbose=@verbose
  END  

  -- Added Txn Logic. 
  IF @tcount = 0
    COMMIT TRANSACTION asisp_createDiscoveryLinks

  RETURN @newlink_rc
GO
