-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_getstartingpaths.sql, tbsm.sql, tbsm_base31e
-- SID: 1.9.2.5
-- Date Modified: 07/12/05 16:40:14
--
-- Module Description:
--   Defines stored procedure asisp_getStartingPaths

USE Object
GO

PRINT 'Creating procedure asisp_getStartingPaths'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_getStartingPaths' AND type = 'P')
  DROP PROCEDURE asisp_getStartingPaths
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_getstartingpaths.sql, tbsm.sql, tbsm_base31e
-- SID: 1.9.2.5
-- Date Modified: 07/12/05 16:40:14
CREATE PROCEDURE asisp_getStartingPaths
  @cid      ClassID,
  @id       ObjID,
  @table    sysname,
  @verbose  TINYINT = 0
AS
  DECLARE @now NVARCHAR(40), @err INT, @rc INT

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_getStartingPaths: @cid=''%s'', @id=%d, @table=%d. Starting', 0, 1, @now, @cid, @id, @table) WITH NOWAIT
  END
  
  -- verify that the table exists in the correct form
  DECLARE @return_val INT, @table_prefix sysname
  IF @table IS NOT NULL
  BEGIN
    IF CHARINDEX('#', @table) = 1
      SET @table_prefix = 'tempdb..'
    ELSE
      SET @table_prefix = ''
    EXEC("
      IF OBJECT_ID(LTRIM('" + @table_prefix + @table + "')) IS NULL
      OR NOT EXISTS (SELECT * FROM " + @table_prefix + "syscolumns AS sc JOIN " + @table_prefix + "sysobjects AS so ON so.id = sc.id WHERE so.id = ISNULL(OBJECT_ID(LTRIM('" + @table_prefix + @table + "')), 0) AND sc.name = 'id')
      OR NOT EXISTS (SELECT * FROM " + @table_prefix + "syscolumns AS sc JOIN " + @table_prefix + "sysobjects AS so ON so.id = sc.id WHERE so.id = ISNULL(OBJECT_ID(LTRIM('" + @table_prefix + @table + "')), 0) AND sc.name = 'cid')
      OR NOT EXISTS (SELECT * FROM " + @table_prefix + "syscolumns AS sc JOIN " + @table_prefix + "sysobjects AS so ON so.id = sc.id WHERE so.id = ISNULL(OBJECT_ID(LTRIM('" + @table_prefix + @table + "')), 0) AND sc.name = 'idpath')
      OR NOT EXISTS (SELECT * FROM " + @table_prefix + "syscolumns AS sc JOIN " + @table_prefix + "sysobjects AS so ON so.id = sc.id WHERE so.id = ISNULL(OBJECT_ID(LTRIM('" + @table_prefix + @table + "')), 0) AND sc.name = 'nonleaf')
        DECLARE exec_cursor INSENSITIVE CURSOR FOR SELECT 0
      ELSE
        DECLARE exec_cursor INSENSITIVE CURSOR FOR SELECT 1
    ")
    OPEN exec_cursor
    FETCH NEXT FROM exec_cursor INTO @return_val
    DEALLOCATE exec_cursor
    IF @return_val = 0 OR @return_val IS NULL
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths: ERROR: Table %s does not exist or does not have the correct columns.', 11, 1, @now, @table) WITH NOWAIT
      RETURN -30000
    END
  END
  ELSE
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_getStartingPaths: ERROR: Parameter @table cannot be NULL.', 11, 1, @now, @table) WITH NOWAIT
    RETURN -30000
  END

  -- temp tables
  CREATE TABLE #GSP_src_obj
  (
    cid      ClassID,
    id       ObjID,
    phys_cid ClassID NULL,
    phys_id  ObjID DEFAULT 0,
    src_cid  ClassID,
    src_id   ObjID,
    nonleaf  TINYINT NOT NULL DEFAULT 0
  )
  CREATE TABLE #GSP_src_idpath
  (
    idpath  NVARCHAR(400) NOT NULL,
    cid     ClassID       NULL,
    id      ObjID         NOT NULL DEFAULT 0,
    nonleaf TINYINT       NOT NULL DEFAULT 1
  )

  -- variable declarations
  DECLARE @locktcount INT
  -- get the latest id processed by the cache for LOBs
  DECLARE @LOB_id ObjID

  -- check the cache for an existing result set if the caller is an LOB or LOBC
  DECLARE @cache_id INT, @last_id ObjID, @cache_completed INT
  SET @cache_completed=-1
  -- the @cache_completed flag has the following meanings
  ----- -1   - proc is attempting to get a cache entry
  ----- -10  - the proc either got a cache entry to build into or failed to get a cache entry, either way, build it and try to enter the results in cache later
  ----- 0    - this proc has responsibility to build the cache
  ----- 1    - this proc will use an existing cache

  -- check the cache for the latest update time
  DECLARE @cache_update DATETIME, @update_leaf TINYINT
  SELECT @cache_update = MAX(StartTime)
  FROM ObjPathCacheRun AS o
  WHERE o.EndTime IS NOT NULL
    AND o.Inserted > 0
  SET @update_leaf = 0

  -- get the latest LOB id
  SET @LOB_id = NULL
  SELECT @LOB_id = id
  FROM ObjPathCacheIDs
  WHERE cid = 'LOB'
  IF @LOB_id IS NULL
    SET @LOB_id = 0

  IF @cid IN ('LOB', 'LOBC')
  BEGIN
    SET @locktcount = 20 -- if after 20 loops we cannot grab an entry in the table go build it anyway
    WHILE @locktcount > 0 AND @cache_completed <> -10 AND (@cache_completed = -1 OR @cache_completed=0 )
    BEGIN  -- -10 means it is mine to build
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_getStartingPaths: Top of while loop looking for a cache entry', 0, 1, @now) WITH NOWAIT
      END
      SET @locktcount = @locktcount-1 
      SET @cache_id = NULL
      SET @cache_completed=-1  
    --- There is always a possibility that the LOBID that will be used is outdates while the
    ---   rest of this process runs.  It is not big deal.  What we do have to be careful of is
    ---   a cache entry being deleted while it is in the process of being used.  We are likely
    ---   going to have to create a inuse COUNTER that is incremented and decremented in this
    ---   proc
      SELECT @cache_id = id,  @cache_completed=completed
      FROM StartingPathCache
      WHERE obj_cid = @cid AND obj_id = @id
        AND last_id = @LOB_id
      ORDER BY id -- this should get the last one

      IF @cache_completed = -1
      BEGIN -- in this case create the entry and mark completed to 0 to mean in progress
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: asisp_getStartingPaths: Creating a cached entry that I will build', 0, 1, @now) WITH NOWAIT
        END
        -- create a new cache entry
        INSERT INTO StartingPathCache (obj_cid, obj_id, last_id, completed)   
        SELECT @cid, @id, @LOB_id, 0 -- 0 indicates the build in in progress
        WHERE NOT EXISTS (
          SELECT id
          FROM StartingPathCache
          WHERE obj_cid=@cid AND obj_id=@id AND last_id=@LOB_id)
        SET @cache_id = @@IDENTITY
        IF @cache_id IS NOT NULL
          SET @cache_completed=-10
      END

      ELSE IF @cache_completed = 0
      BEGIN   --- in this case wait
        IF @locktcount > 0
        BEGIN
          IF @verbose >= 1
          BEGIN
            SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
            RAISERROR('%s: asisp_getStartingPaths: waiting in loop for a cache entry to complete', 0, 1, @now) WITH NOWAIT
          END
          WAITFOR DELAY '00:00:03'  --- sleep for awhile before checking if completed    
          IF @verbose >= 1
          BEGIN
            SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
            RAISERROR('%s: asisp_getStartingPaths: done waiting in loop for a cache entry to complete', 0, 1, @now) WITH NOWAIT
          END
        END
        ELSE
        BEGIN
          IF @verbose >= 1
          BEGIN
            SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
            RAISERROR('%s: asisp_getStartingPaths: timed out waiting for cache entry to complete', 0, 1, @now) WITH NOWAIT
          END
          SET @cache_id = NULL 
        END
      END -- end of @cache_completed = -1
    END -- end of the while cache completed loop 
  END
  ELSE
    SET @cache_id = NULL
  
  DECLARE @MarkedInUse INT
  SET @MarkedInUse=0 
  IF @cache_id IS NOT NULL
  BEGIN  -- increment counter of cache to make sure it does not get deleted on me
    UPDATE StartingPathCache
    SET inuse = inuse+1, @MarkedInUse=1
    WHERE id=@cache_id
  END

  -- if the object is an LOB, get the leaf nodes
  IF @cid IN ('LOB', 'LOBC') AND (@cache_completed = -10 OR @cache_id IS NULL) -- @cache_id should only be null if timeout or physical object
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths: Starting special LOB processing', 0, 1, @now) WITH NOWAIT
    END

    -- get LOB descendents
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Getting LOB descendents', 0, 1, @now) WITH NOWAIT
    END
    INSERT INTO #GSP_src_obj (cid, id, src_cid, src_id)
    SELECT DISTINCT o.cid, o.id, o.src_cid, o.src_id
    FROM ObjPathCache AS o WITH (NOLOCK) 
      JOIN ObjPathCache AS o2 WITH (NOLOCK) ON (o.idpath LIKE o2.idpath + '/%' OR o.idpath = o2.idpath)
    WHERE o2.cid = @cid AND o2.id = @id
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Finished getting %d LOB descendent(s)', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- create indexes
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Creating indexes on #obj_src', 0, 1, @now) WITH NOWAIT
    END
    CREATE CLUSTERED INDEX i#GSP_src_obj ON #GSP_src_obj (cid, id)
    CREATE INDEX i#GSP_src_obj_src ON #GSP_src_obj (src_cid, src_id)
    CREATE INDEX i#GSP_src_obj_phys ON #GSP_src_obj (phys_cid, phys_id)
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Finished creating indexes on #obj_src', 0, 1, @now) WITH NOWAIT
    END

    -- update to indicate leaf nodes
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Computing LOB leaf nodes', 0, 1, @now) WITH NOWAIT
    END
    UPDATE o_src
    SET nonleaf = 1
    FROM #GSP_src_obj AS o
      JOIN #GSP_src_obj AS o_src ON o.src_cid = o_src.cid AND o.src_id = o_src.id
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Finished computing LOB leaf nodes', 0, 1, @now) WITH NOWAIT
    END

    -- get physical objects
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Getting physical objects from LOBs', 0, 1, @now) WITH NOWAIT
    END
    UPDATE o
    SET phys_cid = l.phy_cid, phys_id = l.phy_id
    FROM #GSP_src_obj AS o WITH (NOLOCK)
      JOIN lob_link AS l WITH (NOLOCK) ON l.src_id = 0 AND l.dst_id = o.id
    WHERE o.cid = 'LOB'
      AND l.phy_cid IS NOT NULL AND l.phy_cid <> 'LOB'
    OPTION(MAXDOP 1)
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Finished getting %d physical object(s) from LOBs', 0, 1, @now, @rc) WITH NOWAIT
    END

/*
    -- remove non-physical objects
    RAISERROR('%s: asisp_getStartingPaths:   Deleting LOBs not linked to physical objects', 0, 1, @now) WITH NOWAIT
    DELETE #GSP_src_obj WHERE phys_cid IS NULL
    SET @datetime = GETDATE()
    SET @datetime_str = CONVERT(NVARCHAR(100), @datetime, 121)
    RAISERROR('%s: asisp_getStartingPaths:   Finished deleting LOBs not linked to physical objects', 0, 1, @now) WITH NOWAIT
*/
    -- update LOBs not linked to physical objects
    -- We must do this now since LOBs can have events
    UPDATE #GSP_src_obj
    SET phys_cid = cid, phys_id = id, nonleaf = 1
    WHERE phys_cid IS NULL

    -- get a list of the root objects idpath values
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Getting idpaths of source objects', 0, 1, @now) WITH NOWAIT
    END
    INSERT INTO #GSP_src_idpath (idpath)   
    SELECT DISTINCT o3.idpath
    FROM (
      SELECT o2.cid AS cid, o2.id AS id, MAX(o2.idpath) AS idpath
      FROM ObjPathCache AS o2 WITH (NOLOCK)
        JOIN #GSP_src_obj AS so WITH (NOLOCK) ON so.phys_cid = o2.cid AND so.phys_id = o2.id
      GROUP BY o2.cid, o2.id
    ) AS o3
    OPTION(MAXDOP 1)
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Parsing %d idpaths', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- compute the cid and id
    UPDATE #GSP_src_idpath    ---- 12 secs 100000 entries   24 secs for 200000 entries  58 secs for 500000
    SET cid =
      CASE
      WHEN CHARINDEX('/', idpath)>0
        THEN LEFT(REVERSE(LEFT(REVERSE(idpath), CHARINDEX('/', REVERSE(idpath))-1)), CHARINDEX(':',REVERSE(LEFT(REVERSE(idpath), CHARINDEX('/', REVERSE(idpath))-1)))-1)
      ELSE LEFT(idpath, CHARINDEX(':', idpath)-1 )
      END,
      id = CAST(RIGHT(idpath, CHARINDEX(':', REVERSE(idpath))-1) AS INT) 
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Finished getting source idpaths cid and ids', 0, 1, @now) WITH NOWAIT
    END

    -- compute leaf nodes
    DELETE s2
    FROM #GSP_src_obj AS s1
     JOIN #GSP_src_obj AS s2 ON s1.phys_cid = s2.phys_cid AND s1.phys_id = s2.phys_id AND s1.id < s2.id
    WHERE s2.nonleaf = 1
    CREATE INDEX cidid#GSP_src_idpath ON #GSP_src_idpath(cid, id)
    UPDATE s
    SET s.nonleaf = so.nonleaf
    FROM #GSP_src_idpath AS s
      JOIN (
        SELECT DISTINCT s1.phys_cid, s1.phys_id, CASE s1.nonleaf WHEN 1 THEN ISNULL(s2.nonleaf, 1) ELSE s1.nonleaf END AS nonleaf
        FROM #GSP_src_obj AS s1
          LEFT OUTER JOIN #GSP_src_obj AS s2 ON s1.phys_cid = s2.phys_cid AND s1.phys_id = s2.phys_id AND s1.id < s2.id
      ) AS so ON so.phys_cid = s.cid AND so.phys_id = s.id
    OPTION(MAXDOP 1)
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:      src_idpath ...    done first attempt at updating leaf node count %d', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- do further computing of leaf nodes by checking the cache to see if the objects are parents
    UPDATE s
    SET s.nonleaf = 2
    FROM #GSP_src_idpath AS s WITH (NOLOCK)
    WHERE NOT EXISTS (
      SELECT src_id
      FROM ObjPathCache AS opc WITH (NOLOCK)
      WHERE opc.src_cid = s.cid AND opc.src_id = s.id
    )
      AND s.nonleaf = 0
    OPTION(MAXDOP 1)
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:        src_idpath ... done last attempt at updating leaf node count %d', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- add the primary key to #src_idpath
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Creating index on #src_idpath', 0, 1, @now) WITH NOWAIT
    END
    CREATE CLUSTERED INDEX idpath#GSP_src_idpath ON #GSP_src_idpath(idpath,nonleaf)
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Finished creating index on #src_idpath', 0, 1, @now) WITH NOWAIT
    END
    -- delete objects from this table whose parents are already in the table
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Deleting objects whose parents are in the list', 0, 1, @now) WITH NOWAIT
    END
    DELETE s1
    FROM #GSP_src_idpath AS s1
      JOIN #GSP_src_idpath AS s2 ON s1.idpath LIKE s2.idpath + '/%'
    WHERE s2.nonleaf = 0
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths:   Finished deleting %d object(s) whose parents are in the list', 0, 1, @now, @rc) WITH NOWAIT
      RAISERROR('%s: asisp_getStartingPaths: Finished special LOB processing', 0, 1, @now) WITH NOWAIT
    END

  END
  ELSE IF @cache_id IS NULL
    INSERT INTO #GSP_src_idpath (cid, id, idpath, nonleaf)
    SELECT cid, id, idpath, 0
    FROM ObjPathCache WITH (NOLOCK)
    WHERE cid = @cid AND id = @id
  ELSE
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths: Using cached paths (id=[%d]).', 0, 1, @now, @cache_id) WITH NOWAIT
    END
    INSERT INTO #GSP_src_idpath (cid, id, idpath, nonleaf)
    SELECT cid, id, idpath, nonleaf
    FROM StartingPathCacheDetail
    WHERE cache_id = @cache_id

    -- update paths in case physical resources have been deleted
    UPDATE o
    SET o.idpath = o2.idpath
    FROM #GSP_src_idpath AS o  WITH (NOLOCK)
      JOIN ObjPathCache AS o2 WITH (NOLOCK) ON o.cid = o2.cid AND o.id = o2.id

    -- verify that the nonleaf status is up-to-date
    IF @cid = 'LOB' AND @cache_update > (SELECT ctime FROM StartingPathCache WITH (NOLOCK) WHERE id = @cache_id)
    BEGIN
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_getStartingPaths: Recomputing leaf node status.', 0, 1, @now) WITH NOWAIT
      END
      -- first, reset the nonleaf status of 2 back to 0
      UPDATE #GSP_src_idpath
      SET nonleaf = 0
      WHERE nonleaf = 2
      -- recompute the nonleaf status based on the cache
      UPDATE s
      SET s.nonleaf = 2
      FROM #GSP_src_idpath AS s WITH (NOLOCK)
      WHERE 
        NOT EXISTS (
          SELECT src_id
          FROM ObjPathCache AS opc WITH (NOLOCK)
          WHERE opc.src_cid = s.cid AND opc.src_id = s.id
        )
        AND s.nonleaf = 0
    END
  END

  -- put the results into the table
  EXEC(
    'INSERT INTO ' + @table + ' (cid, id, idpath, nonleaf) ' +
    'SELECT cid, id, idpath, CASE nonleaf WHEN 0 THEN 0 ELSE 1 END ' +
    'FROM #GSP_src_idpath '
  )
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_getStartingPaths: Finished putting results in incoming table', 0, 1, @now) WITH NOWAIT
  END

  -- put the clustered index on the table
  EXEC(
    'CREATE CLUSTERED INDEX idpath' + @table + ' ON ' + @table + '(idpath, nonleaf) '-- try this instead of primary key 
  )
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_getStartingPaths: Finished creating clustered index on return table', 0, 1, @now) WITH NOWAIT 
  END

  -- cache the results
  IF (@cache_id IS NULL AND @cid IN ('LOB', 'LOBC')) OR (@cache_completed=-10)
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths: Starting to cache results ', 0, 1, @now) WITH NOWAIT
    END

    -- create a new cache entry
    IF @cache_completed<>-10
    BEGIN   --- this really should not happen or only happen in rare condidtions
      INSERT INTO StartingPathCache (obj_cid, obj_id, last_id, completed)
      VALUES ( @cid, @id, @LOB_id, 0 )
      -- get the entry number
      SET @cache_id=@@IDENTITY
    END
    IF @cache_id IS NULL
    BEGIN
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_getStartingPaths: ERROR: Could not get a cache id from StartingPathCache for cid=[%s] and id=[%d].', 11, 1, @now, @cid, @id) WITH NOWAIT
      END
      RETURN -30000
    END

    -- place the results into the cache
    INSERT INTO StartingPathCacheDetail (cache_id, cid, id, idpath, nonleaf)
    SELECT @cache_id, cid, id, idpath, nonleaf
    FROM #GSP_src_idpath

    -- update the latest id value  THIS CODE SHOULD REALLY JUST USE THE LOB_ID IT GOT EARLIER
    UPDATE spc
    SET spc.last_id = @LOB_id, completed=1
    FROM StartingPathCache AS spc
    WHERE spc.id = @cache_id
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_getStartingPaths: Finished saving cached results', 0, 1, @now) WITH NOWAIT
    END
  END

  IF @cache_id IS NOT NULL AND @MarkedInUse=1
  BEGIN  --- increment counter of cache to make sure it does not get deleted on me
    UPDATE StartingPathCache SET inuse = inuse-1 WHERE id=@cache_id
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_getStartingPaths: Finished', 0, 1, @now) WITH NOWAIT
  END
GO
