-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/hist_physevents.sql, tbsm_base, tbsm_base31e
-- SID: 1.13.5.13
-- Date Modified: 08/01/22 05:23:46
--
-- Module Description:
--   Defines the hist_PhysEvents stored proc which IS used by the Reporting System.
-- History of Changes:
---  OW57191  - LOB REPORTS TIMEOUT FROM REPORTING SYSTEM.
--   OA09872  - update deleted objects with * indicator.
--   PMR25706 - The mach and name column in #objtemp was increased in size from ObjName - NVARCHAR(54) to NVARCHAR(255)
--              This was required for deleted resources with names that were already at the max length of 54.
--              Attempting to pre-pend the '*' character onto deleted resource name would generate a "String
--              or bianry data would be truncated" error in the Reporting System WEB application.
--   DCR117   - query extended length attribute and do event search filter if event search text is not empty
--   OA18672  - changed the whereclause for detail & msgname to what it was before DCR 117 and
--              added a whereclause for extended_length_attr_filter similar to detail
USE Object
GO

-------------------------------------------------------------------------------------------
PRINT 'Creating PROCEDURE hist_PhysEvents.'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'hist_PhysEvents' AND type = 'P')
  DROP PROCEDURE hist_PhysEvents
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/hist_physevents.sql, tbsm_base, tbsm_base31e
-- SID: 1.13.5.13
-- Date Modified: 08/01/22 05:23:46
--
-- Module Description:
--   Defines the hist_PhysEvents stored proc which IS used by the Reporting System.
-- History of Changes:
--   OW57191 - LOB REPORTS TIMEOUT FROM REPORTING SYSTEM
--   OA09872 - update deleted objects with * indicator
--   PMR25706 - The mach and name column in #objtemp was increased in size from ObjName - NVARCHAR(54) to NVARCHAR(255)
--              This was required for deleted resources with names that were already at the max length of 54.
--              Attempting to pre-pend the '*' character onto deleted resource name would generate a "String
--              or bianry data would be truncated" error in the Reporting System WEB application.
--
CREATE PROCEDURE hist_PhysEvents
  @cid ClassID,
  @id ObjID,
  @alertstates NVARCHAR(255),
  @priorities NVARCHAR(255),
  @begtime NVARCHAR(30),
  @endtime NVARCHAR(30),
  @eventtypes NVARCHAR(255),
  @orderopt NCHAR(1),
  @detail NVARCHAR(255),
  @IncludeChild TINYINT=1,
  @machineflag TINYINT=0,
  @nullstatepri NVARCHAR(50) = '(Unknown)',
  @msgname NVARCHAR(255)='',
  @extended_length_attr_filter NVARCHAR(255) -- DCR117
AS

SET NOCOUNT ON

CREATE TABLE #objtemp (cid ClassID, id ObjID, mach NVARCHAR(255) NULL, name NVARCHAR(255) NULL, machElementNum int default 0 not null, machPosition int default 0 not null , trailingDelimPos int default 0 not null)

IF @IncludeChild=1
  EXEC report_phys_objects @cid, @id 
ELSE
  INSERT #objtemp (cid, id) SELECT @cid, @id

EXEC report_addclassdata '#objtemp', 'name', 'name'

IF @machineflag=1
BEGIN

  EXEC report_addclassdata '#objtemp', 'mach', 'name', 'MACH'
  EXEC report_addclassdata '#objtemp', 'mach', 'name', 'UNIX'
  EXEC report_addclassdata '#objtemp', 'mach', 'name', 'NTSR'
  EXEC report_addclassdata '#objtemp', 'mach', 'name', 'HANT'
  EXEC report_addclassdata '#objtemp', 'mach', 'name', 'HAUX'

END

DECLARE @loop INT
DECLARE @maxLoop INT

IF @machineflag = 1
BEGIN
  -- Calculate the element number for the MACHINE portion of the path.
  UPDATE t
  SET t.machElementNum = (LEN(SUBSTRING(idpath,0,CHARINDEX('MACH:',idpath))) - 
  LEN(REPLACE(SUBSTRING(idpath,0,CHARINDEX('MACH:',idpath)), '/', '')))/LEN('/') 
  FROM #objtemp t INNER JOIN ObjPathCache o ON o.cid = t.cid and o.id = t.id

  SELECT @loop = 1
  SELECT @maxLoop = max (machElementNum) from #objtemp

  --Find the position of last '/' before the MACHINE name
  WHILE ( @loop <= @maxLoop )
  BEGIN 
    UPDATE t
    SET t.machPosition = CHARINDEX('/', resourcepath, machPosition) + 1
    FROM #objtemp t INNER JOIN ObjPathCache o ON o.cid = t.cid and o.id = t.id
    WHERE machElementNum >= @loop

    SELECT @loop = @loop + 1
  END

  --Find the position of first '/' after the MACHINE name
  UPDATE t
  SET t.trailingDelimPos = CHARINDEX('/', resourcepath, machPosition)
  FROM #objtemp t INNER JOIN ObjPathCache o ON o.cid = t.cid and o.id = t.id
  WHERE machElementNum > 0

  --For MACHINE resources, the MACHINE name is the last name. so no trailing /.
  UPDATE t
  SET t.trailingDelimPos = len(resourcepath) + 1
  FROM #objtemp t INNER JOIN ObjPathCache o ON o.cid = t.cid and o.id = t.id
  WHERE machElementNum > 0 and trailingDelimPos = 0

  --Extract the MACHINE name from the resourcepath
  UPDATE t
  SET t.mach  = SUBSTRING(resourcepath, machPosition, trailingDelimPos-machPosition)
  FROM #objtemp t INNER JOIN ObjPathCache o ON o.cid = t.cid and o.id = t.id
  WHERE machElementNum > 0
END

-- Make sure there are no NULL machine names returned.
-- This will prevent the Active report from hanging.
UPDATE #objtemp SET mach = '(N/A)' WHERE mach IS NULL

-- flag each object with an * in the report that is marked deleted
DECLARE @viewname VARCHAR(30)
DECLARE @sql VARCHAR(1000)
DECLARE temp_cursor CURSOR FOR 
SELECT DISTINCT cid FROM #objtemp
OPEN temp_cursor
FETCH NEXT FROM temp_cursor INTO @cid
WHILE @@FETCH_STATUS = 0
BEGIN
  SELECT @viewname = cname + '_V' FROM obj_class WHERE cid = @cid
  SET @sql = 'UPDATE o SET o.name = "*" + o.name FROM #objtemp o 
              JOIN ' + @viewname + ' v ON v.id = o.id 
              WHERE o.cid = "' + @cid +'" AND v.deleted = 1'
  EXEC(@sql)
  FETCH NEXT FROM temp_cursor INTO @cid
END
CLOSE temp_cursor
DEALLOCATE temp_cursor

DECLARE @column_list NVARCHAR(255), @table_list NVARCHAR(255), @where_clause NVARCHAR(255)
DECLARE @orderby NVARCHAR(255), @where_clause2 NVARCHAR(255), @where_clause3 NVARCHAR(255)
-- dcr117 changed @whereClause from 255 to 600 in case both detail and extended attribute have 255 characters
DECLARE @where_clause4 NVARCHAR(600)
SELECT @where_clause2='', @where_clause3='', @where_clause4=''

IF @machineflag=1
  SELECT @column_list='machsys=isnull(o.mach,''(N/A)''),'
ELSE
  SELECT @column_list=''

SELECT @column_list=@column_list + 'objtype=o.cid, objname=o.name, deleted=0, eventtype=e.cid,alertstate=isnull(ast._Desc,''' + @nullstatepri + '''),priority=isnull(op._Priority,''' + @nullstatepri + '''), ident=e.name,msgtext=e.detail,DATETIME=e.ctime'

-- OA19932: since event table on primary db server does not have the extended_attr column (DCR 117), 
-- need to send in "" for it for customers running reporting system solely off primary db server
DECLARE @tableid INT
SELECT @tableid = id FROM sysobjects WHERE name = 'event'
IF EXISTS (SELECT * FROM syscolumns WHERE id = @tableid AND name = 'extended_attr')
  SELECT @column_list = @column_list + ',e.extended_attr' --DCR117
ELSE
  SELECT @column_list = @column_list + ',""'

-- Code added by OW57191
SELECT @table_list='#objtemp o'
IF EXISTS (SELECT * FROM Object..sysobjects where name = 'event' and type = 'U')
   SELECT @table_list=@table_list+', event e (index=event_PK)'
ELSE
   BEGIN
     IF EXISTS (SELECT * FROM Object..sysobjects where name = 'event' and type = 'V')
        BEGIN
           DECLARE @vid INT
           DECLARE @text NVARCHAR(255)
           SELECT @vid=id FROM Object..sysobjects where name = 'event' and type = 'V'
           SELECT @text=text FROM Object..syscomments where id=@vid
           SELECT @vid=CHARINDEX('eventhist',@text)
           IF @vid > 0  -- BCP approach installed
              SELECT @table_list=@table_list+', History..eventhist e (index=event_PK)'
             ELSE       -- Linked server approach installed.
              SELECT @table_list=@table_list+', event e (index=event_PK)'
        END
     ELSE SELECT @table_list=@table_list+', event e (index=event_PK)'  -- event not 'U' or 'V' ... unknown type
   END
SELECT @table_list=@table_list+', SixLevelPri_A op, AlertState_A ast'

-- End Code added by OW57191
-- code replaced by OW57191
-- SELECT @table_list='#objtemp o,event e (index=event_PK),SixLevelPri_A op,AlertState_A ast'
--

SELECT @where_clause = 'o.cid=e.obj_cid AND o.id=e.obj_id AND op.id=e.priority AND ast.id=e.alertstate'
IF @alertstates <> ''
BEGIN
  IF charindex(',',@alertstates) > 0
    SELECT @where_clause2=@where_clause2 + ' AND e.alertstate IN (' + @alertstates + ')'
  ELSE
    SELECT @where_clause2=@where_clause2 + ' AND e.alertstate=' + @alertstates
END
IF @priorities <> ''
BEGIN
  IF charindex(',',@priorities) > 0
    SELECT @where_clause2=@where_clause2 + ' AND e.priority IN (' + @priorities + ')'
  ELSE
    SELECT @where_clause2=@where_clause2 + ' AND e.priority=' + @priorities
END
IF @eventtypes<>''
BEGIN
  IF charindex(',',@eventtypes) > 0
    SELECT @where_clause2=@where_clause2 + ' AND e.cid IN (' + @eventtypes + ')'
  ELSE
    SELECT @where_clause2=@where_clause2 + ' AND e.cid=' + @eventtypes
END

SELECT @where_clause3=" AND e.ctime>=CONVERT(DATETIME,'" + @begtime + "',101) AND e.ctime<CONVERT(DATETIME,'" + @endtime + "',101)"
IF @msgname <> ''
  --dcr117 the % signs are taken care of before the parameter is passed in.
  -- the search supports wildcard characters
  SELECT @where_clause3=@where_clause3 + ' AND e.name like N''%' + @msgname + '%'''
  -- SELECT @where_clause3=@where_clause3 + " AND e.name like N'" + @msgname + "'" -- APAR OA18672

IF @detail <> ''
  --dcr117 the % signs are taken care of before the parameter is passed in.
  -- the search supports wildcard character
  SELECT @where_clause4=' AND e.detail like N''%' + @detail + '%''' 
  -- SELECT @where_clause4=" AND e.detail like '" + @detail + "' "  -- APAR OA18672

-- DCR117, the event search string should be converted to sql form before passing in
-- For example, if a user enters *tec event * into the search input in report,
-- the search string must be converted to %tec event% and pass this converted string into this procedure
--IF @extended_length_attr_filter <> ""
--   SELECT @where_clause4 = @where_clause4 + " AND e.extended_attr LIKE '" + @extended_length_attr_filter + "' "
IF @extended_length_attr_filter <> ''
     SELECT @where_clause4 = @where_clause4 + ' AND e.extended_attr LIKE N''%' + @extended_length_attr_filter + '%'''   -- APAR OA18672

IF @orderopt = '1'
  SELECT @orderby='e.ctime'
ELSE
BEGIN
  IF @machineflag=1
    SELECT @orderby='o.mach,o.cid,o.name,e.ctime'
  ELSE
    SELECT @orderby='o.cid,o.name,e.ctime'
END

-- debug 
--PRINT 'Select Statement := ' + 'SELECT ' + @column_list + ' FROM ' + @table_list + ' WHERE ' + @where_clause + @where_clause2 + @where_clause3 + @where_clause4 + ' ORDER BY ' + @orderby


EXEC('SET forceplan ON
  SELECT ' + @column_list + ' FROM ' + @table_list + ' WHERE ' + @where_clause + @where_clause2 + @where_clause3 + @where_clause4 + ' ORDER BY ' + @orderby)

SET NOCOUNT OFF
GO
