-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 2005,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/tep_setup.sql, tbsm_base, tbsm_base31e
-- SID: 1.2
-- Date Modified: 07/11/28 07:47:07
--
-- Module Description
--   Defines the tep_setup stored procedure

USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tep_setup' AND type = 'P')
  DROP PROCEDURE tep_setup
GO

PRINT 'Creating procedure tep_setup'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/tep_setup.sql, tbsm_base, tbsm_base31e
-- SID: 1.2
-- Date Modified: 07/11/28 07:47:07
-- tep_setup is run from SQL Analyser to configure the TBSM TEP Feed
CREATE PROCEDURE tep_setup
  @action        NVARCHAR(20) = NULL,      -- List|Add|Delete|Refresh|Help.   'List' shows current settings
  @soap_server   ObjName      = NULL,      -- TEMS Soap Server tcp/ip name or address
  @soap_alias    NVARCHAR(20) = NULL,      -- TEMS Soap Server Alias name.  Default is 'soap'.  If a TEMS SOAP Server Hub is defined, specify the ALIAS here (see install doc).
  @soap_uid      NVARCHAR(20) = NULL,      -- (Optional) Userid to be used to query TEMS via SOAP.
                                           --   This is only required if the target TEMS has been configured with Security: Validate User option On.  The TEMS default is Off.
  @soap_pw       NVARCHAR(20) = NULL,      -- (Optional) Password for the @soap_uid		
  @soap_protocol NVARCHAR(10) = NULL,      -- Protocol for SOAP requests.  http | https. Default is http. 
  @soap_port     INT          = NULL,      -- TEMS SOAP ip port for http/https SOAP communications. 1920 default matches TEMS SOAP default.
  @teps_server   ObjName      = NULL,      -- TEPS tcp/ip name or address.  Require for
  @teps_type     NVARCHAR(20) = NULL,      -- TEPS database type.  DB2 | MSSQL
  @teps_db       NVARCHAR(20) = NULL,      -- TEPS database name.  'TEPS' is commonly used for ITM v6.1.  'cnps' is commonly used for OMEGAMON XE.
  @teps_uid	 NVARCHAR(20) = NULL,      -- Userid to be used to read TEPS tables
  @teps_pw	 NVARCHAR(20) = NULL,      -- Password for @teps_uid. Note: Case sensitive.
  @teps_protocol NVARCHAR(10) = NULL,      -- Protocol for URL launch to TEP browser.  http | https. Default is http. 
  @teps_port     INT          = NULL,      -- TEPS ip port for TEP browser. 1920 default matches TEPS default.
  @tec_events    TINYINT      = NULL,      -- @tec_events=1 means events are received via TEC instead of SOAP (SOAP is still used for discovery though and this will populate CL_IDCache too).
  @anchor_physical  ObjName   = NULL,      -- Name of Aggregate object in physical tree to anchor TEP discovery
  @anchor_business  ObjName   = NULL,      -- (Optional) Name of Business System (LOB) object in business system tree to anchor TEP DE discovery
  @verbose       TINYINT = 0,
  @db2_port_num  INT = 50000               -- (Optional) Port number for DB2 linked TEPS server. Default is 50000.
AS

  SET ANSI_DEFAULTS OFF
  SET CONCAT_NULL_YIELDS_NULL OFF
  SET NOCOUNT ON

  DECLARE @rc INT

------------------------------------------------------------------------------------
--  Validate parms
------------------------------------------------------------------------------------
  SET @action = CASE UPPER(LEFT(@action,1)) 
                  WHEN 'L' THEN 'LIST'
                  WHEN 'A' THEN 'ADD'
                  WHEN 'P' THEN 'PASSWORD'
                  WHEN 'D' THEN 'DELETE'
                  WHEN 'R' THEN 'REFRESH'
                  WHEN 'T' THEN 'TEST'
                  WHEN 'H' THEN 'HELP'
                  WHEN '?' THEN 'HELP'
                  ELSE @action END

  IF ISNULL(@action,'') = ''
  BEGIN
    RAISERROR('tep_setup: Require @action parameter.  Valid values are LIST, ADD, PASSWORD, DELETE, REFRESH, TEST or HELP',3,1,@action) WITH NOWAIT
    RAISERROR("  Use tep_setup 'HELP' for more details.",0,1) WITH NOWAIT
    RETURN -30000
  END
  IF @action NOT IN ('LIST','ADD', 'PASSWORD', 'DELETE', 'REFRESH', 'TEST','HELP')
  BEGIN
    RAISERROR('tep_setup: Invalid @action "%s".  Must be LIST, ADD, PASSWORD, DELETE, REFRESH, TEST or HELP',3,1,@action) WITH NOWAIT
    RAISERROR("  Use tep_setup 'HELP' for more details.",0,1) WITH NOWAIT
    RETURN -30000
  END

------------------------------------------------------------------------------------
-- Process @action with subprocs
------------------------------------------------------------------------------------
  IF @action = 'ADD'
    EXEC @rc = tep_setup_add @soap_server=@soap_server, @soap_alias=@soap_alias, 
                             @soap_protocol=@soap_protocol, @soap_port=@soap_port, 
                             @soap_uid=@soap_uid, @soap_pw=@soap_pw, 
                             @teps_server=@teps_server, @teps_type=@teps_type, @teps_db=@teps_db, 
                             @teps_uid=@teps_uid, @teps_pw=@teps_pw, 
                             @teps_protocol=@teps_protocol, @teps_port=@teps_port,
                             @tec_events=@tec_events,
                             @anchor_physical=@anchor_physical, @anchor_business=@anchor_business,
                             @verbose=@verbose, @db2_port_num = @db2_port_num
  ELSE IF @action = 'DELETE'
    EXEC @rc = tep_setup_delete   @soap_server=@soap_server, @soap_alias=@soap_alias, @verbose=@verbose
  ELSE IF @action = 'PASSWORD'
    EXEC @rc = tep_setup_password @soap_server=@soap_server, @soap_alias=@soap_alias, 
                                  @soap_uid=@soap_uid, @soap_pw=@soap_pw, 
                                  @teps_server=@teps_server, 
                                  @teps_uid=@teps_uid, @teps_pw=@teps_pw,
                                  @verbose=@verbose 
  ELSE IF @action = 'LIST'
    EXEC @rc = tep_setup_list     @verbose=@verbose
  ELSE IF @action = 'TEST'
    EXEC @rc = tep_setup_test     @verbose=@verbose
  ELSE IF @action = 'REFRESH'
    EXEC @rc = tep_setup_refresh  @verbose=@verbose
  ELSE 
    EXEC @rc = tep_setup_help     @verbose=@verbose


  RETURN @rc
GO
