-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 2005,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/tep_setup_add.sql, tbsm_base, tbsm_base31e
-- SID: 1.3
-- Date Modified: 07/11/28 07:46:21
--
-- Module Description
--   Defines the tep_setup_add stored procedure

USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tep_setup_add' AND type = 'P')
  DROP PROCEDURE tep_setup_add
GO

PRINT 'Creating procedure tep_setup_add'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/tep_setup_add.sql, tbsm_base, tbsm_base31e
-- SID: 1.3
-- Date Modified: 07/11/28 07:46:21
-- tep_setup_add is invoked from tep_setup to to configure the TBSM TEP Feed
CREATE PROCEDURE tep_setup_add
  @soap_server   ObjName      = NULL,      -- TEMS Soap Server tcp/ip name or address
  @soap_alias    NVARCHAR(20) = NULL,      -- TEMS Soap Server Alias name.  Default is 'soap'.  If a TEMS SOAP Server Hub is defined, specify the ALIAS here (see install doc).
  @soap_uid      NVARCHAR(20) = NULL,      -- (Optional) Userid to be used to query TEMS via SOAP.
                                           --   This is only required if the target TEMS has been configured with Security: Validate User option On.  The TEMS default is Off.
  @soap_pw       NVARCHAR(20) = NULL,      -- (Optional) Password for the @soap_uid		
  @soap_protocol NVARCHAR(10) = NULL,      -- Protocol for SOAP requests.  http | https. Default is http. 
  @soap_port     INT          = NULL,      -- TEMS SOAP ip port for http SOAP communications. 1920 default matches TEMS SOAP default.
  @teps_server   ObjName      = NULL,      -- TEPS tcp/ip name or address.  Require for
  @teps_type     NVARCHAR(20) = NULL,      -- TEPS database type.  DB2 | MSSQL
  @teps_db       NVARCHAR(20) = NULL,      -- TEPS database name. 'TEPS' is commonly used for ITM v6.1.  'cnps' is commonly used for OMEGAMON XE.
  @teps_uid	 NVARCHAR(20) = NULL,      -- Userid to be used to read TEPS tables
  @teps_pw	 NVARCHAR(20) = NULL,      -- Password for @teps_uid. Note: Case sensitive.
  @teps_protocol NVARCHAR(10) = NULL,      -- Protocol for URL launch to TEP browser.  http | https. Default is http. 
  @teps_port     INT          = NULL,      -- TEPS ip port for CNP browser. 1920 default matches TEPS default.
  @tec_events    TINYINT      = NULL,      -- @tec_events=1 means events are received via TEC instead of SOAP (SOAP is still used for discovery though and this will populate CL_IDCache too).
  @anchor_physical     ObjName      = NULL,      -- Name of Aggregate object in physical tree to anchor TEP discovery
  @anchor_business     ObjName      = NULL,      -- (Optional) Name of Business System (LOB) object in business system tree to anchor TEP DE discovery
  @verbose       TINYINT = 0,
  @db2_port_num  INT = 50000               -- (Optional) Port number for DB2 linked TEPS server. Default is 50000.
AS

  SET ANSI_DEFAULTS OFF
  SET CONCAT_NULL_YIELDS_NULL OFF
  SET NOCOUNT ON

  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT

  DECLARE @rc INT, @err INT, @sql NVARCHAR(4000), @log_text NVARCHAR(1000)

  DECLARE @refresh_excp_id ObjID
  DECLARE @linked_server NVARCHAR(100), @teps_from_clause NVARCHAR(100), @teps_provider NVARCHAR(20)
  DECLARE @output_error NVARCHAR(500), @soap_request NVARCHAR(4000), @url NVARCHAR(200), @obj_id ObjID
  DECLARE @job_name sysname, @job_id UNIQUEIDENTIFIER, @ret INT, @job_status NVARCHAR(20)
  DECLARE @job_catname sysname, @job_desc NVARCHAR(255), @job_cmd NVARCHAR(1000), @enabled TINYINT
  DECLARE @wait INT, @ent_id ObjID, @ent_name ObjName
  DECLARE @soap_hdr_proc NVARCHAR(32)
  DECLARE @provstr NVARCHAR(1000)

  CREATE TABLE #RLSESTAMP (RLSESTAMP NVARCHAR(50) NULL)
  CREATE TABLE #Name (Name NVARCHAR(64) NULL)
  CREATE TABLE #cmdout (line NVARCHAR(500) NULL, rowno INT IDENTITY(1,1) )

------------------------------------------------------------------------------------
--  Validate/Init parms
------------------------------------------------------------------------------------
  -- defaults
  IF ISNULL(@soap_alias,'') = ''         SET @soap_alias       = 'soap'
  IF @soap_uid IS NULL                   SET @soap_uid         = ''
  IF @soap_pw  IS NULL OR @soap_uid = '' SET @soap_pw          = ''
  IF ISNULL(@soap_protocol,'') = ''      SET @soap_protocol    = 'http'
  IF ISNULL(@soap_port,'') = ''          SET @soap_port        = 1920
  IF ISNULL(@teps_protocol,'') = ''      SET @teps_protocol    = 'http'
  IF ISNULL(@teps_port,'') = ''          SET @teps_port        = 1920
  IF @tec_events IS NULL                 SET @tec_events       = 0
  IF ISNULL(@anchor_physical,'') = ''    SET @anchor_physical  = 'TEP Physical'
  IF @anchor_business  IS NULL           SET @anchor_business  = 'TEP Business'

  -- standardize case
  SET @soap_protocol = LOWER(@soap_protocol)
  SET @teps_protocol = LOWER(@teps_protocol)
  IF @teps_type IS NOT NULL SET @teps_type = UPPER(@teps_type) 
  IF EXISTS(SELECT * 
              FROM tep_config 
              WHERE LOWER(anchor_physical) = LOWER(anchor_physical)   -- match case insensitive
                AND anchor_physical <> anchor_physical)               -- no match for same case
  BEGIN                                                   
    SELECT @anchor_physical = anchor_physical
      FROM tep_config 
      WHERE LOWER(anchor_physical) = LOWER(@anchor_physical)
        AND anchor_physical <> @anchor_physical
    RAISERROR('tep_setup_add: NOTE - Overriding anchor_physical to ''%s'' to match case of existing anchor_physical.',0,1,@anchor_physical) WITH NOWAIT
  END
  IF EXISTS(SELECT * 
              FROM tep_config 
              WHERE LOWER(anchor_business) = LOWER(@anchor_business)   -- match case insensitive
                AND anchor_business <> @anchor_business)               -- no match for same case
  BEGIN                                                   
    SELECT @anchor_business = anchor_business
      FROM tep_config 
      WHERE LOWER(anchor_business) = LOWER(@anchor_business)
        AND anchor_business <> @anchor_business
    RAISERROR('tep_setup_add: NOTE - Overriding anchor_business to ''%s'' to match case of existing anchor_business.',0,1,@anchor_business) WITH NOWAIT
  END

  -- check for pre-reqs
  DECLARE @missing NVARCHAR(500)
  SET @missing = ''
  IF ISNULL(@soap_server,'') = '' SET @missing = @missing+', @soap_server'  
  IF ISNULL(@teps_server,'') = '' SET @missing = @missing+', @teps_server'  
  IF ISNULL(@teps_type,'')   = '' SET @missing = @missing+', @teps_type'  
  IF ISNULL(@teps_db,'')     = '' SET @missing = @missing+', @teps_db'  
  IF ISNULL(@teps_uid,'')    = '' SET @missing = @missing+', @teps_uid'  
  IF @teps_pw     IS NULL         SET @missing = @missing+', @teps_pw'  

  IF @missing <> ''
  BEGIN
    SET @missing = SUBSTRING(@missing,3,LEN(@missing)-2)  -- rid leading comma
    RAISERROR("tep_set_add:  Missing required parameter(s) - %s",3,1,@missing) WITH NOWAIT
    RAISERROR("  Use tep_setup 'HELP' for more details.",0,1) WITH NOWAIT
    RETURN -30000
  END

  -- check invalid parms
  IF @soap_protocol NOT IN ('http','https')
  BEGIN
    RAISERROR('tep_setup_add: ERROR - Invalid @soap_protocol "%s".  Must be ''http'' or ''https''.',3,1,@soap_protocol) WITH NOWAIT
    RAISERROR("  Use tep_setup 'HELP' for more details.",0,1) WITH NOWAIT
    RETURN -30000
  END
  IF @teps_type NOT IN ('DB2','MSSQL')
  BEGIN
    RAISERROR('tep_setup_add: ERROR - Invalid @teps_type "%s".  Must be ''DB2'' or ''MSSQL''.',3,1,@teps_type) WITH NOWAIT
    RAISERROR("  Use tep_setup 'HELP' for more details.",0,1) WITH NOWAIT
    RETURN -30000
  END
  IF @teps_protocol NOT IN ('http','https')
  BEGIN
    RAISERROR('tep_setup_add: ERROR - Invalid @teps_protocol "%s".  Must be ''http'' or ''https''.',3,1,@teps_protocol) WITH NOWAIT
    RAISERROR("  Use tep_setup 'HELP' for more details.",0,1) WITH NOWAIT
    RETURN -30000
  END
  IF @soap_pw = '********' OR @teps_pw = '********' 
  BEGIN
    RAISERROR('tep_setup_add: ERROR - Invalid password ''********''.',3,1) WITH NOWAIT
    RAISERROR("  Use tep_setup 'HELP' for more details.",0,1) WITH NOWAIT
    RETURN -30000
  END
  IF EXISTS(SELECT * FROM tep_config 
              WHERE LOWER(soap_server) = LOWER(@soap_server) AND LOWER(soap_alias) = LOWER(@soap_alias) )
  BEGIN
    RAISERROR("tep_setup_add: There is already an existing definition for @soap_server='%s', @soap_alias='%s'.",3,1,@soap_server, @soap_alias) WITH NOWAIT
    RAISERROR("  Use tep_setup 'HELP'   for more details.",0,1) WITH NOWAIT
    RAISERROR("  Use tep_setup 'LIST'   to list existing definitions.",0,1) WITH NOWAIT
    RAISERROR("  Use tep_setup 'DELETE' to delete an existing definition.",0,1) WITH NOWAIT
    RETURN -30000
  END
  IF @anchor_business <> ''
  AND EXISTS(SELECT *
               FROM tep_config 
               WHERE anchor_business = @anchor_business
                 AND anchor_physical <> anchor_physical)
  BEGIN                                                   
    RAISERROR('tep_setup_add: ERROR - @anchor_business=''%s'' and anchor_physical=''%s'' conflict with an existing definition that has a different anchor_physical.',3,1,@anchor_business,@anchor_physical) WITH NOWAIT
    RAISERROR("  Use tep_setup 'HELP'   for more details.",0,1) WITH NOWAIT
    RAISERROR("  Use tep_setup 'LIST'   to list existing definitions.",0,1) WITH NOWAIT
    RAISERROR("  Use tep_setup 'DELETE' to delete an existing definition.",0,1) WITH NOWAIT
    RETURN -30000
  END

  -- echo/log parms
  SET @log_text = '@soap_server='+@soap_server+', @soap_alias='+@soap_alias
               +', @soap_uid='+@soap_uid+', @soap_pw='+@soap_pw
               +', @soap_protocol='+@soap_protocol+', @soap_port='+CAST(@soap_port AS NVARCHAR)
               +', @teps_server='+@teps_server+', @teps_type='+@teps_type+', @teps_db='+@teps_db
               +', @teps_uid='+@teps_uid+', @teps_pw='+@teps_pw+
               +', @teps_protocol='+@teps_protocol+', @teps_port='+CAST(@teps_port AS NVARCHAR)
               +', @tec_events='+CAST(@tec_events AS NVARCHAR)
               +', @anchor_physical='+@anchor_physical+', @anchor_business='+@anchor_business
  EXEC tep_log @proc_name='tep_setup_add', @log_text=@log_text
  RAISERROR("tep_setup_add %s",0,1,@log_text) WITH NOWAIT
  PRINT ''


------------------------------------------------------------------------------------
--  Init vars
------------------------------------------------------------------------------------
  SET @job_catname   = 'Tivoli Enterprise Portal Monitoring'
  SET @linked_server = 'teps_'+@teps_server
  SET @teps_provider = CASE WHEN @teps_type = 'DB2' THEN 'IBMDADB2' ELSE 'SQLOLEDB' END
   
  DECLARE @tbsm_InstallDirectory NVARCHAR(1000)
  EXEC master.dbo.xp_regread 'HKEY_LOCAL_MACHINE',
                'SOFTWARE\Accessible Software, Inc.\Access1\1.0',
                'InstallDirectory',
                @tbsm_InstallDirectory OUTPUT
  SET @tbsm_InstallDirectory = RTRIM(@tbsm_InstallDirectory)
  IF RIGHT(@tbsm_InstallDirectory,1) = '\'
    SET @tbsm_InstallDirectory = RTRIM(LEFT(@tbsm_InstallDirectory,LEN(@tbsm_InstallDirectory)-1))
  IF @tbsm_InstallDirectory IS NULL
  BEGIN
    RAISERROR('tep_setup_add: ERROR - Could not read SOFTWARE\Accessible Software, Inc.\Access1\1.0\InstallDirectory registry setting',3,1) WITH NOWAIT
    RETURN -30000
  END


--***************************************************************************************************
--  Add the config
--***************************************************************************************************

------------------------------------------------------------------------------------
--  Locate/Create anchor_physical
------------------------------------------------------------------------------------
  SET @obj_id = NULL
  SELECT @obj_id = MIN(a.id), @anchor_physical = a.name
    FROM Aggregate_C AS a
    WHERE LOWER(a.name) = LOWER(@anchor_physical)
      AND a.deleted = 0
    GROUP BY a.name
  IF @obj_id IS NOT NULL
  BEGIN
    RAISERROR("OK: Anchor for TEP physical discovery '%s' already exists - AGGR:%d",0,1,@anchor_physical,@obj_id) WITH NOWAIT
  END
  ELSE
  BEGIN
    SELECT @ent_id = id, @ent_name = name
      FROM ENT_C 
      WHERE deleted = 0
    SET @rc = @@ROWCOUNT
    IF @rc > 1
    BEGIN
      RAISERROR("ERROR: Cannot create anchor_physical Aggreate object '%s' since there are multiple Enterprise objects.",3,1,@anchor_physical) WITH NOWAIT
      RAISERROR("       From the TBSM Console, manually insert the anchor_physical Aggregate object '%s' under one of the Enterprise objects.",0,1,@anchor_physical) WITH NOWAIT
      RETURN -30000
    END
    ELSE IF @rc = 0
    BEGIN
      RAISERROR("ERROR: Could not locate an Enterprise object.",3,1) WITH NOWAIT
      RAISERROR("       From the TBSM Console, manually insert an Enterprise object.",0,1) WITH NOWAIT
      RETURN -30000
    END
    ELSE
    BEGIN
      EXEC @obj_id = asisp_createAGGR @name=@anchor_physical, @parent_cid = 'ENT', @parent_id = @ent_id, @Desc='Anchor point for TEP discovery'
      IF @obj_id IS NOT NULL
      BEGIN
        RAISERROR("OK: Anchor for TEP Physical discovery '%s' was created under Enterprise '%s' - AGGR:%d",0,1,@anchor_physical, @ent_name, @obj_id) WITH NOWAIT
      END
      ELSE
      BEGIN
        RAISERROR("ERROR: Anchor for TEP Physical discovery '%s' could not be created",3,1,@anchor_physical) WITH NOWAIT
        RETURN -30000
      END
    END
  END

------------------------------------------------------------------------------------
--  Locate/Create anchor_business
------------------------------------------------------------------------------------
  IF ISNULL(@anchor_business,'') <> ''
  BEGIN
    SET @obj_id = NULL
    SELECT @obj_id = MIN(lob.id), @anchor_business = lob.name
      FROM LineOfBusiness_V AS lob
      JOIN lob_link         AS ll ON ll.dst_id = lob.id
      WHERE lob.deleted = 0 AND lob._DeletedCID = '' AND LOWER(lob.name) = LOWER(@anchor_business)
        AND ll.src_id = 0 AND phy_cid IS NULL
      GROUP BY lob.name
    IF @obj_id IS NOT NULL
    BEGIN
      RAISERROR("OK: Anchor for TEP Business discovery '%s' already exists - LOB:%d",0,1,@anchor_business, @obj_id) WITH NOWAIT
    END
    ELSE
    BEGIN
      EXEC @obj_id = asisp_createLOB @name=@anchor_business, @parent_cid = 'LOBC', @parent_id = 1, @Desc='Anchor point for TEP DE discovery'
      IF @obj_id IS NOT NULL
      BEGIN
        RAISERROR("OK: Anchor for TEP Business discovery '%s' was created at root level - LOB:%d ",0,1,@anchor_business,@obj_id) WITH NOWAIT
      END
      ELSE
      BEGIN
        RAISERROR("ERROR: Anchor for TEP Business discovery '%s' could not be created",3,1,@anchor_business) WITH NOWAIT
        RETURN -30000
      END
    END
  END  -- anchor_business

------------------------------------------------------------------------------------
--  Add linked server
------------------------------------------------------------------------------------
  IF EXISTS(SELECT * FROM master..sysservers WHERE srvname = @linked_server)
  BEGIN
    EXEC @rc = sp_dropserver @server=@linked_server, @droplogins='droplogins'
    IF @rc <> 0
    BEGIN
      RAISERROR("ERROR: Failed to remove old TEPS linked server %s.",3,1,@linked_server) WITH NOWAIT  
      RETURN -30000
    END
    RAISERROR("OK: Successfully removed old TEPS linked server %s.",0,1,@linked_server) WITH NOWAIT  
  END

  SET @rc = -30000
  IF @teps_type = 'DB2'
  BEGIN
    IF @db2_port_num <> 50000
    BEGIN
      SELECT @provstr = 'Hostname='+@teps_server+';Database='+@teps_db+';Protocol=TCPIP;Port='+convert(nvarchar(10),@db2_port_num)

      EXEC @rc = sp_addlinkedserver @server=@linked_server,
                                    @srvproduct='',
                                    @provstr=@provstr,
                                    @provider=@teps_provider
    END
    ELSE
    BEGIN
      EXEC @rc = sp_addlinkedserver @server=@linked_server,
                                    @srvproduct='',
                                    @datasrc=@teps_db,
                                    @location=@teps_server,
                                    @provider=@teps_provider
    END
  END
  ELSE -- MS SQL
    EXEC @rc = sp_addlinkedserver @server=@linked_server,
                                  @srvproduct='',
                                  @datasrc=@teps_server,
                                  @catalog=@teps_db,
                                  @provider=@teps_provider
  IF @rc <> 0
  BEGIN
    RAISERROR('ERROR: sp_addlinkedserver failed with rc %d for %s %s',3,1,@rc,@linked_server,@teps_provider) WITH NOWAIT
    RETURN -30000
  END

  EXEC @rc = sp_addlinkedsrvlogin @rmtsrvname=@linked_server,
                                  @useself='false',
                                  @rmtuser=@teps_uid,
                                  @rmtpassword=@teps_pw
  IF @rc <> 0
  BEGIN
    RAISERROR('ERROR: sp_addlinkedsrvlogin failed with rc %d for %s',3,1,@rc,@linked_server) WITH NOWAIT
    RETURN -30000
  END
   
  EXEC sp_serveroption @server=@linked_server, @optname='data access',    @optvalue='true'  
  EXEC sp_serveroption @server=@linked_server, @optname='rpc',            @optvalue='true'  
  EXEC sp_serveroption @server=@linked_server, @optname='rpc out',        @optvalue='true'  
  EXEC sp_serveroption @server=@linked_server, @optname='connect timeout',@optvalue='10'
  EXEC sp_serveroption @server=@linked_server, @optname='query timeout',  @optvalue='10'
  RAISERROR("OK: Successfully defined a linked server for %s.",0,1,@linked_server) WITH NOWAIT  

  PRINT ''
  RAISERROR("Testing access to TEPS database '%s' ....",0,1, @teps_server) WITH NOWAIT
  IF @teps_type = 'DB2'
  BEGIN
    TRUNCATE TABLE #cmdout
    INSERT INTO #cmdout EXEC master..xp_cmdshell 'whereis IBMDADB2.DLL'
    IF NOT EXISTS(SELECT * FROM #cmdout WHERE UPPER(line) LIKE '%IBMDADB2.DLL%' )
    BEGIN 
      RAISERROR('WARNING: Could not locate IBMDADB2.DLL in the Path.  This may mean that the DB2 Run-Time client is not installed.',0,1) WITH NOWAIT
    END
    RAISERROR('    If access to TEPS fails, check:',0,1) WITH NOWAIT
    RAISERROR('     - Parameters @teps_type=''%s'', @teps_db=''%s'', @teps_uid=''%s'', @teps_pw=''%s''.',0,1,@teps_type, @teps_db, @teps_uid, @teps_pw) WITH NOWAIT
    RAISERROR('     - The DB2 Run-Time Client is installed with OLE DB Support (IBMDADB2)',0,1) WITH NOWAIT
    RAISERROR('     - Use tep_setup ''HELP'' for more information about the DB2 Run-Time client, including where to download it from.',0,1) WITH NOWAIT
  END
  ELSE -- MSSQL
  BEGIN
    RAISERROR('    If access to TEPS fails, check parameters @teps_type=''%s'', @teps_db=''%s'', @teps_uid=''%s'', @teps_pw=''%s''.',0,1,@teps_type, @teps_db, @teps_uid, @teps_pw) WITH NOWAIT
  END

  SET @teps_from_clause = NULL
  CREATE TABLE #priv (TABLE_CAT sysname NULL, TABLE_SCHEM sysname NULL, TABLE_NAME sysname NULL, GRANTOR sysname NULL, GRANTEE sysname NULL, PRIVILEGE varchar(32) NULL, IS_GRANTABLE varchar(3) NULL)
  INSERT INTO #priv EXEC sp_table_privileges_ex @table_server=@linked_server , @table_name='KFWDBVER', @table_catalog=@teps_db
  SELECT @teps_from_clause = '['+@linked_server+'].'+ISNULL(TABLE_CAT,'')+'.'+TABLE_SCHEM+'.'
    FROM #priv
  IF @teps_from_clause IS NULL
  BEGIN
    RAISERROR('ERROR: No results from sp_table_privileges_ex @table_server=%s, @table_name=''KFWDBVER'' ',3,1,@linked_server) WITH NOWAIT
    RETURN -30000
  END

  TRUNCATE TABLE #RLSESTAMP
  SET @sql = "
    SET ANSI_NULLS ON
    SET ANSI_WARNINGS ON
    EXEC('INSERT INTO #RLSESTAMP SELECT RLSESTAMP FROM "+@teps_from_clause+"KFWDBVER ')
  "
--PRINT @sql
  EXEC(@sql)
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @rc = 0
  BEGIN
    RAISERROR("ERROR: Could not access TEPS database on '%s'",3,1,@teps_server) WITH NOWAIT
    RETURN -30000
  END 
  RAISERROR("OK: Successfully tested access to TEPS database '%s'.",0,1, @teps_server) WITH NOWAIT

------------------------------------------------------------------------------------
--  Add tep_config entry
------------------------------------------------------------------------------------
  IF @tcount = 0
    BEGIN TRANSACTION tep_setup_add
  ELSE
    SAVE TRANSACTION tep_setup_add

  ---------------------------------------------------------
  -- Insert entry into tep_config
  ---------------------------------------------------------
  INSERT INTO tep_config (soap_server, soap_alias, soap_uid, soap_protocol, soap_port, 
                          teps_server, teps_type, teps_uid, teps_protocol, teps_port,
                          tec_events, enabled, anchor_physical, anchor_business, teps_from_clause)
    SELECT @soap_server, @soap_alias, @soap_uid, @soap_protocol, @soap_port, 
           @teps_server, @teps_type, @teps_uid, @teps_protocol, @teps_port, 
           @tec_events, 1, @anchor_physical, @anchor_business, @teps_from_clause
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    RAISERROR("ERROR: Insert to tep_config failed with error code %d for %s %s.",3,1,@err, @soap_server, @soap_alias) WITH NOWAIT 
    ROLLBACK TRANSACTION tep_setup_add
    RETURN -30000
  END
  ELSE IF @rc = 0
  BEGIN
    RAISERROR("ERROR: No entry was added into tep_config table for %s %s.",3,1,@soap_server, @soap_alias) WITH NOWAIT
    ROLLBACK TRANSACTION tep_setup_add
    RETURN -30000
  END

  ---------------------------------------------------------
  -- create tep_soap_hdr_<config id> proc, which is used by tep_soap and tep_events_config
  ---------------------------------------------------------
  EXEC tep_setup_add_soap_hdr @soap_server=@soap_server, @soap_alias=@soap_alias, @soap_uid=@soap_uid, @soap_pw=@soap_pw

  ---------------------------------------------------------
  -- check soap comms
  ---------------------------------------------------------
  PRINT ''
  RAISERROR("Testing access to TEMS SOAP server '%s' alias '%s' ....",0,1, @soap_server, @soap_alias) WITH NOWAIT
  SET @soap_request = '<CT_Get>'
                        +'<object>ManagedSystem</object>'
                        +'<afilter>Product;EQ;EM</afilter>'
                       +'</CT_Get>'
  TRUNCATE TABLE #Name
  EXEC @rc = tep_soap @soap_server  = @soap_server,
                      @soap_alias   = @soap_alias,
                      @soap_request = @soap_request,
                      @output_table = '#Name',
                      @output_error = @output_error OUTPUT,
                      @verbose      = @verbose
  IF @output_error <> '' 
  BEGIN
    RAISERROR("ERROR: Could not access soap_server '%s' soap_alias '%s'.  %s. ",3,1,@soap_server, @soap_alias, @output_error) WITH NOWAIT
    ROLLBACK TRANSACTION tep_setup_add
    RETURN -30000
  END
  RAISERROR("OK: Successfully tested access to TEMS SOAP server '%s' alias '%s'.",0,1, @soap_server, @soap_alias) WITH NOWAIT

  ---------------------------------------------------------
  -- Add SQL Job Category
  ---------------------------------------------------------
  PRINT ''
  RAISERROR('Defining SQL Task Jobs ....',0,1,@job_name) WITH NOWAIT
  IF NOT EXISTS (SELECT * FROM msdb..syscategories WHERE name = @job_catname)
  BEGIN
    EXEC @rc = msdb..sp_add_category @name=@job_catname
    IF @rc <> 0
    BEGIN
      RAISERROR('ERROR: Could not add SQL Task Category %s',3,1,@job_catname) WITH NOWAIT
      ROLLBACK TRANSACTION tep_setup_add
      RETURN -30000
    END
    RAISERROR('OK: Added SQL Task Category %s',0,1,@job_catname) WITH NOWAIT
  END

  ---------------------------------------------------------
  -- Add SQL Task job 'TEP Discovery' 
  ---------------------------------------------------------
  SET @job_id = NULL
  SET @job_name = 'TEP Discovery'
  SET @job_desc = 'Query TEMS for TEP discovery.  Query TEPS for TEP Business Discovery.' 
  SET @job_cmd  = 'tep_disc'
--EXEC msdb..sp_delete_job @job_name=@job_name
  IF NOT EXISTS (SELECT * FROM msdb..sysjobs WHERE name = @job_name)
  BEGIN
    -- define job
    EXEC @rc = msdb..sp_add_job @job_name=@job_name, @category_name=@job_catname, @owner_login_name='sa',
                                @description = @job_name, @job_id = @job_id OUTPUT
    IF @rc <> 0
    BEGIN
      RAISERROR('ERROR: Could not add SQL Task Job %s',3,1,@job_name) WITH NOWAIT
      ROLLBACK TRANSACTION tep_setup_add
      RETURN -30000
    END
    -- define job step
    EXEC @rc = msdb..sp_add_jobstep @job_id=@job_id, @subsystem='TSQL', @command=@job_cmd, 
                                    @database_name='Object', @step_name = 'Discover'
    IF @rc <> 0
    BEGIN
      RAISERROR('ERROR: Could not add SQL Task Job Step to Job %s',3,1,@job_name) WITH NOWAIT
      ROLLBACK TRANSACTION tep_setup_add
      RETURN -30000
    END
    -- Associate the job with a server
    EXEC @rc = msdb..sp_add_jobserver @job_name=@job_name, @server_name=@@SERVERNAME
    IF @rc <> 0
    BEGIN
      RAISERROR('ERROR: Could not associate the job with a server SQL Task Job %s with a server',3,1,@job_name) WITH NOWAIT
      ROLLBACK TRANSACTION tep_setup_add
      RETURN -30000
    END
    -- Define schedule for which to run the job
    EXEC @rc = msdb..sp_add_jobschedule @name='Every 1 hour', @job_name=@job_name, @enabled=1,
                                        @freq_type=4,                -- run daily
                                        @freq_interval=1,            -- run every day
                                        @freq_subday_type=8,         -- hours
                                        @freq_subday_interval=1      -- every 1 hour
    IF @rc <> 0
    BEGIN
      RAISERROR('ERROR: Could not added SQL Task schedule for Job %s',3,1,@job_name) WITH NOWAIT
      ROLLBACK TRANSACTION tep_setup_add
      RETURN -30000
    END
    RAISERROR('OK: Added SQL Task Job %s',0,1,@job_name) WITH NOWAIT
  END -- end job definition for Discovery
  ELSE
  BEGIN
    RAISERROR("OK: SQL Task Job '%s' is already defined",0,1,@job_name) WITH NOWAIT
  END

  ---------------------------------------------------------
  -- Start TEP Discovery job
  ---------------------------------------------------------
  EXEC tep_job_status @job_name=@job_name, @job_status=@job_status OUTPUT
  IF @job_status = 'enabled'
  BEGIN
    RAISERROR("Starting SQL Task Job '%s' ....",0,1,@job_name) WITH NOWAIT
    EXEC @rc = msdb..sp_start_job @job_name = @job_name
  END
  ELSE
  BEGIN
    RAISERROR("NOTE: SQL Task Job '%s' is currently %s" ,0,1,@job_name, @job_status) WITH NOWAIT
  END

  ---------------------------------------------------------
  -- Add SQL Task job 'TEP Receive Events '+@soap_server+' '+@soap_alias
  ---------------------------------------------------------
  SET @job_id = NULL
  SET @job_name = 'TEP Receive Events '+@soap_server+' '+@soap_alias
  SET @job_desc = 'Receive events from TEP SOAP server '+@soap_server+' using SOAP alias '+@soap_alias+'.'
                 +'  NOTE: The job will only stop ''Executing'' if there is a break in communications with the SOAP Server.'
                 +'  The job''s schedule interval is used to retry communications to the SOAP Server'
  SET @job_cmd  = 'cscript "'+@tbsm_InstallDirectory+'\bin\tep_events.js"'
                 +' "'+@soap_server+'" '
                 +' "'+@soap_alias+'" '
                 +' //NoLogo'
--  EXEC msdb..sp_delete_job @job_name=@job_name
  IF NOT EXISTS (SELECT * FROM msdb..sysjobs WHERE name = @job_name)
  BEGIN
    -- define job
    EXEC @rc = msdb..sp_add_job @job_name=@job_name, @category_name=@job_catname, @owner_login_name='sa',
                                @description=@job_desc, @job_id=@job_id OUTPUT
    IF @rc <> 0
    BEGIN
      RAISERROR('ERROR: Could not add SQL Task Job %s',3,1,@job_name) WITH NOWAIT
      ROLLBACK TRANSACTION tep_setup_add
      RETURN -30000
    END
    -- define job step 1
    DECLARE @reset_jobstatus NVARCHAR(2000)
    SET @reset_jobstatus = "
      UPDATE js SET last_run_outcome = 1
        FROM msdb..sysjobs       AS j
        JOIN msdb..sysjobservers AS js ON js.job_id = j.job_id
        WHERE j.name = '"+@job_name+"'
          AND js.last_run_outcome <> 1
      "
    EXEC @rc = msdb..sp_add_jobstep @job_id=@job_id, @subsystem='TSQL', @command=@reset_jobstatus, 
                                    @on_success_action=3,  -- go to next step
                                    @database_name='Object', @step_name = 'Reset job status icon'
    IF @rc <> 0
    BEGIN
      RAISERROR('ERROR: Could not add SQL Task Job Step 1 to Job %s',3,1,@job_name) WITH NOWAIT
      ROLLBACK TRANSACTION tep_setup_add
      RETURN -30000
    END
    -- define job step 2
    EXEC @rc = msdb..sp_add_jobstep @job_id=@job_id, @subsystem='CMDEXEC', @command=@job_cmd,
      		@database_name='Object', @step_name='Receive events'
    IF @rc <> 0
    BEGIN
      RAISERROR('ERROR: Could not add SQL Task Job Step 2 to Job %s',3,1,@job_name) WITH NOWAIT
      ROLLBACK TRANSACTION tep_setup_add
      RETURN -30000
    END
    -- Associate the job with a server
    EXEC @rc = msdb..sp_add_jobserver @job_name=@job_name, @server_name=@@SERVERNAME
    IF @rc <> 0
    BEGIN
      RAISERROR('ERROR: Could not associate the job with a server SQL Task Job %s with a server',3,1,@job_name) WITH NOWAIT
      ROLLBACK TRANSACTION tep_setup_add
      RETURN -30000
    END
    -- Define schedule for which to run the job
    EXEC @rc = msdb..sp_add_jobschedule @name='Every 1 minute', @job_name=@job_name, @enabled=1,
                                        @freq_type=4,                -- run daily
                                        @freq_interval=1,            -- run every day
                                        @freq_subday_type=4,         -- minutes
                                        @freq_subday_interval=1      -- every 1 minute
    IF @rc <> 0
    BEGIN
      ROLLBACK TRANSACTION tep_setup_add
      RAISERROR('ERROR: Could not add SQL Task schedule for Job %s',3,1,@job_name) WITH NOWAIT
      RETURN -30000
    END
    RAISERROR('OK: Added SQL Task Job %s',0,1,@job_name) WITH NOWAIT
    END -- end job definition for Events
  ELSE
  BEGIN
    RAISERROR("OK: SQL Task Job '%s' is already defined",0,1,@job_name) WITH NOWAIT
  END

  ---------------------------------------------------------
  -- Start receiving events
  ---------------------------------------------------------
  EXEC tep_job_status @job_name=@job_name, @job_status=@job_status OUTPUT
  IF @job_status = 'running'
  BEGIN
    RAISERROR('NOTE: Stopping current running SQL Task Job %s ....',0,1,@job_name) WITH NOWAIT
    EXEC @rc = msdb..sp_stop_job @job_name = @job_name
  END
  ELSE IF @job_status <> 'enabled'
  BEGIN
    RAISERROR('NOTE: SQL Task Job ''%s'' is currently %s',0,1,@job_name, @job_status) WITH NOWAIT
  END

  ---------------------------------------------------------
  -- Add SQL Task job 'TEP Cleanup'
  ---------------------------------------------------------
  SET @job_id = NULL
  SET @job_name = 'TEP Cleanup'
  SET @job_desc = 'Maintenance job to cleanup logs etc'
  SET @job_cmd  = "tep_log @action='PURGE', @purge_days=3"
--EXEC msdb..sp_delete_job @job_name=@job_name
  IF NOT EXISTS (SELECT * FROM msdb..sysjobs WHERE name = @job_name)
  BEGIN
    -- define job
    EXEC @rc = msdb..sp_add_job @job_name=@job_name, @category_name=@job_catname, @owner_login_name='sa',
                @description=@job_name, @job_id=@job_id OUTPUT
    IF @rc <> 0
    BEGIN
      RAISERROR('ERROR: Could not add SQL Task Job %s',3,1,@job_name) WITH NOWAIT
      ROLLBACK TRANSACTION tep_setup_add
      RETURN -30000
    END
    -- define job step
    EXEC @rc = msdb..sp_add_jobstep @job_id=@job_id, @subsystem='TSQL', @command=@job_cmd, 
      		@database_name='Object', @step_name = 'Cleanup tep_debuglog'
    IF @rc <> 0
    BEGIN
      RAISERROR('ERROR: Could not add SQL Task Job Step to Job %s',3,1,@job_name) WITH NOWAIT
      ROLLBACK TRANSACTION tep_setup_add
      RETURN -30000
    END
    -- Associate the job with a server
    EXEC @rc = msdb..sp_add_jobserver @job_name=@job_name, @server_name=@@SERVERNAME
    IF @rc <> 0
    BEGIN
      RAISERROR('ERROR: Could not associate the job with a server SQL Task Job %s with a server',3,1,@job_name) WITH NOWAIT
      ROLLBACK TRANSACTION tep_setup_add
      RETURN -30000
    END
    -- Define schedule for which to run the job
    EXEC @rc = msdb..sp_add_jobschedule @name='Every 1 day', @job_name=@job_name, @enabled=1,
                                        @freq_type=4,                -- run daily
                                        @freq_interval=1,            -- run every day
                                        @active_start_time='010000'  -- run 01:00 AM (HHMMSS) 
    IF @rc <> 0
    BEGIN
      RAISERROR('ERROR: Could not add SQL Task schedule for Job %s',3,1,@job_name) WITH NOWAIT
      ROLLBACK TRANSACTION tep_setup_add
      RETURN -30000
    END
    RAISERROR('OK: Added SQL Task Job %s',0,1,@job_name) WITH NOWAIT
  END -- end job definition for Cleanup


  IF @tcount = 0
    COMMIT TRANSACTION tep_setup_add

  PRINT ''
  RAISERROR("OK: tep_setup_add complete.",0,1) WITH NOWAIT


GO





