-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 2005,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/tep_setup_help.sql, tbsm_base, tbsm_base31e
-- SID: 1.2
-- Date Modified: 07/11/29 10:47:54
--
-- Module Description
--   Defines the tep_setup_help stored procedure

USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tep_setup_help' AND type = 'P')
  DROP PROCEDURE tep_setup_help
GO

PRINT 'Creating procedure tep_setup_help'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/tep_setup_help.sql, tbsm_base, tbsm_base31e
-- SID: 1.2
-- Date Modified: 07/11/29 10:47:54
-- tep_setup_help is run from SQL Analyser to configure the TBSM TEP Feed
CREATE PROCEDURE tep_setup_help
  @verbose       TINYINT = 0
AS

  SET ANSI_DEFAULTS OFF
  SET CONCAT_NULL_YIELDS_NULL OFF
  SET NOCOUNT ON

  PRINT 'The TBSM Tivoli Enterprise Portal (TEP) Feed is administered by the tep_setup stored procedure.'
  PRINT ''
  PRINT 'Examples:'
  PRINT "   tep_setup @action='ADD', @soap_server='pthtiv20.ibm.com', @soap_alias='pthtic1'"
  PRINT "                          , @soap_uid='tbsmsoap', @soap_pw='tbsmsoap'"
  PRINT "                          , @teps_server='pthtiv25', @teps_type='DB2', @teps_db='TEPS'"
  PRINT "                          , @teps_uid='TEPS', @teps_pw='itm61pw' "
  PRINT ''
  PRINT "   tep_setup @action='ADD', @soap_server='pthtiv20.ibm.com', @soap_alias='pthtic1'"
  PRINT "                          , @soap_uid='tbsmsoap', @soap_pw='tbsmsoap'"
  PRINT "                          , @teps_server='pthtiv25', @teps_type='DB2', @teps_db='TEPS'"
  PRINT "                          , @teps_uid='TEPS', @teps_pw='itm61pw', @verbose=2, @db2_port_num=50001 "
  PRINT ''
  PRINT "   tep_setup @action='ADD', @soap_server='markpres', @soap_alias='soap'"
  PRINT "                          , @soap_uid='tbsmsoap', @soap_pw='tbsmsoap'"
  PRINT "                          , @teps_server='markpres', @teps_type='MSSQL', @teps_db='cnps'"
  PRINT "                          , @teps_uid='sa', @teps_pw='sa_markpres'"
  PRINT ''
  PRINT "   tep_setup @action='LIST' "
  PRINT "   tep_setup @action='TEST' "
  PRINT "   tep_setup @action='REFRESH' "
  PRINT "   tep_setup @action='PASSWORD', @teps_uid='db2admin', @teps_pw='ibm' "
  PRINT "   tep_setup @action='DEL', @soap_server='pthtiv20', @soap_alias='pthtic1' "
  PRINT "   tep_setup @action='DEL', @soap_server='ALL'"
  PRINT ''
  PRINT 'Valid parameters for tep_setup are:'
  PRINT '   @action          = ''LIST''     - Lists the current configuration and diagnosis information'
  PRINT '                      ''ADD''      - Add a new definition for a TEP (TEMS & TEPS) to be monitored by TBSM'
  PRINT '                      ''PASSWORD'' - Change password for @soap_uid and/or @teps_uid'
  PRINT '                      ''DELETE''   - Delete a previously added definition'
  PRINT '                      ''TEST''     - Test the current configuration'
  PRINT '                      ''REFRESH''  - Refresh discovery and events from TEP'
  PRINT '                      ''HELP''     - Show this help information'
  PRINT ''
  PRINT 'The following parameters are for @action = ''ADD'':'
  PRINT '   @soap_server     = TEMS SOAP Server tcp/ip name or address.  See Notes below for more details.'
  PRINT '   @soap_alias      = TEMS SOAP Server Alias.  ''soap'' is the default.'
  PRINT '   @soap_uid        = Userid to be used to query TEMS via SOAP.'
  PRINT '                        This is only needed if the target TEMS has been configured'
  PRINT '                        with "Security: Validate User" option On. '
  PRINT '   @soap_pw         = Password for the @soap_uid.'
  PRINT '   @soap_port       = TEMS SOAP Server TCP/IP port.'
  PRINT '                        1920 is the default, which matches the TEMS SOAP Server default.'
  PRINT '   @teps_server     = TEPS tcp/ip name or address.'
  PRINT '   @teps_type       = TEPS database type.  ''DB2'' | ''MSSQL'''
  PRINT '   @teps_db         = TEPS database name.  ''TEPS'' is commonly used for ITM v6.1.  ''cnps'' is commonly used for OMEGAMON XE.'
  PRINT '   @teps_uid        = Userid to be used to read TEPS tables.'
  PRINT '   @teps_pw         = Password for @teps_uid.  Note: Case sensitive.'
  PRINT '   @teps_port       = TEPS Console TCP/IP port.'
  PRINT '                        1920 is the default, which matches the TEPS default.'
  PRINT '   @tec_events      = 0 (default) means events are received via SOAP.'
  PRINT '                      1 means events are received via TEC instead of SOAP.'
  PRINT '                        SOAP is still used for TEP discovery though and this will populate CL_IDCache too.'
  PRINT '   @anchor_physical = Name of Aggregate object in physical tree to anchor TEP Physical discovery.'
  PRINT '                        Default is ''TEP Physical''.'
  PRINT '                        If the specified Aggregate object does not already exist, it will be'
  PRINT '                        auto-created under the Enterprise object.'
  PRINT '   @anchor_business = Name of Business System object to anchor TEP Business discovery.'
  PRINT '                        Default is ''TEP Business''.'
  PRINT '                        If you do not want TEP Business Discovery, specify @anchor_business=''''.'
  PRINT '                        If the specified Business System object does not already exist, it will be'
  PRINT '                        auto-created as a top level Business System.'
  PRINT '   @verbose         = Level of output to be returned by the procedure. '
  PRINT '                        Default = 0. Options are 0, 1 or 2 with 2 being the most verbose.' 
  PRINT '   @db2_port_num    = Port used for TEPS database only when the TEPS database type is DB2.'
  PRINT '                        50000 is the default, which matches the DB2 default.'
  PRINT ''
  PRINT 'The following parameters are for @action = ''DELETE'':'
  PRINT '   @soap_server  = TEMS SOAP Server tcp/ip name or address, as defined in previous ADD.'
  PRINT '                      Specify ''ALL'' to delete the whole TBSM TEP Feed configuration,'
  PRINT '                      including SQL Tasks' 
  PRINT '   @soap_alias   = TEMS SOAP Server Alias, as defined in previous ADD'
  PRINT ''
  PRINT 'The following parameters are for @action = ''PASSWORD'':'
  PRINT '   @soap_uid      = Userid to be used to query TEMS via SOAP.'
  PRINT '   @soap_pw       = Password for the @soap_uid.'
  PRINT '   @teps_uid      = Userid to be used to read TEPS tables.'
  PRINT '   @teps_pw       = Password for @teps_uid.  Note: Case sensitive.'
  PRINT ''
  PRINT 'Notes:'
  PRINT '  The TBSM TEP Feed supports IBM Tivoli Monitoring (ITM v6.1 and above) and OMEGAMON XE/DE (CMS v350, CNPS v196).'
  PRINT '  The following terms are equivalent:'
  PRINT '    -  Tivoli Enterprise Portal            (TEP ) = CandleNet Portal         (CNP) '
  PRINT '    -  Tivoli Enterprise Portal Server     (TEPS) = CandleNet Portal Server  (CNPS)'
  PRINT '    -  Tivoli Enterprise Management Server (TEMS) = Candle Management Server (CMS) '
  PRINT ''
  PRINT '  In an TEP environment, the Hub TEMS is a central point for all events and the TEPS database'
  PRINT '  contains event definitions and TEP Business View group information.'
  PRINT ''
  PRINT '  The TBSM TEP Feed communicates with the Hub TEMS via a TEMS SOAP Server, and the'
  PRINT '  TEPS database via a DB2 Client or MS SQL Client session.'
  PRINT ''
  PRINT '  tep_setup is used to define the TEMS and TEPS to TBSM.'
  PRINT '  If multiple TEP environments are to be monitored, tep_setup @action=''ADD'' must be used to define'		 
  PRINT '  each TEP environment.  The soap_server + soap_alias combination is the unique key to each configuration.'
  PRINT ''
  PRINT '  tep_setup @action=''ADD'' will create SQL Task jobs to perform discovery and receive events.'
  PRINT ''
  PRINT '  To view TEP discovered objects, right click the Aggregate anchor object (or Enterprise object etc) and'
  PRINT '  Open the ''TEP Resources...'' screen.  Press F5 to refresh the screen.'
  PRINT '  Objects listed can be drag&dropped into Business Systems.'
  PRINT ''
  PRINT '  TEMS SOAP Sever'
  PRINT '  ---------------'
  PRINT '  Before using tep_setup, you must configure a TEMS to be a SOAP Server (Web Services).'
  PRINT ''
  PRINT '  If the Hub TEMS is running on Windows:'
  PRINT '    - For ITM v6.1, SOAP is already enabled.'
  PRINT '    - For OMEGAMON XE, set the ''SOAP Server'' CMS configuration option On.'
  PRINT '    - Specify the Hub TEMS ip name/address in the @soap_server parameter and @soap_alias=''soap''.'
  PRINT ''
  PRINT '  If the Hub TEMS is running on zOS or UNIX:'
  PRINT '    - For ITM v6.1, SOAP is already enabled.'
  PRINT '      - Specify the Hub TEMS ip name/address in the @soap_server parameter and @soap_alias=''soap''.'
  PRINT ''
  PRINT '    - For OMEGAMON XE on these platforms, SOAP is not supported.'
  PRINT '      - Install a CMS on a Windows server (such as the server where the CNPS runs)'
  PRINT '        and configure it as a ''SOAP Hub Server'' using the Advanced configuration dialog.'
  PRINT '        This will involve defining an ALIAS, which can be any name you choose (except ''soap'')'
  PRINT '        and the Hub CMS ip address/name.'
  PRINT '        The TBSM TEP Feed will use this SOAP Server as a gateway to the Hub CMS.'
  PRINT '      - Specify the windows CMS SOAP Server ip name/address in the @soap_server parameter'
  PRINT '        and ALIAS name (that was defined in the SOAP Hub) in the @soap_alias parameter.'
  PRINT ''
  PRINT '  Only the Hub TEMS is defined using tep_setup. Coverage of Remote TEMS will automatically be handled via the Hub TEMS.'
  PRINT ''
  PRINT '  TEPS DB2'
  PRINT '  ---------------'
  PRINT '  If the TEPS is using DB2, you must install the DB2 Run-Time client with OLE DB Support (IBMDADB2) on the'
  PRINT '  TBSM database server before running tep_setup. The DB2 Run Time Client can be downloaded from'
  PRINT '    http://www-306.ibm.com/software/data/db2/udb/support/downloadv8_windows32bit.html'
  PRINT ''
  PRINT '  There are several fixpack versions of the DB2 Run-Time client and in some cases these will only work if the DB2 Server'
  PRINT '  is running the same fixpack.  If tep_setup fails to access the TEPS database due to incompatibility of'
  PRINT '  DB2 fixpacks, compare the IBMDADB2.DLL file directory date stamp on the TBSM database server (DB2 Client) and'
  PRINT '  the TEPS server (DB2 Server).  If they do not match, install the appropriate DB2 Run-time Client fixpack to match the'
  PRINT '  Server fixpack version.  The following are some common date stamps for IBMDADB2.DLL:'
  PRINT '     - 24-OCT-2002 = DB2 v8 GA'
  PRINT '     - 02-FEB-2003 = DB2 v8 FP1'
  PRINT '     - 09-MAY-2003 = DB2 v8 FP2'
  PRINT '     - 23-JAN-2004 = DB2 v8 FP4a'
  PRINT '     - 20-FEB-2004 = DB2 v8 FP5'
  PRINT '     - 15-SEP-2004 = DB2 v8 FP6a'
  PRINT '     - 20-MAY-2005 = DB2 v8 FP6c'
  PRINT '     - 21-MAY-2005 = DB2 v8 FP7b'
  PRINT '     - 26-MAY-2005 = DB2 v8 FP9a'
  PRINT '     - 07-SEP-2005 = DB2 v8 FP10'
  RAISERROR('',0,1) WITH NOWAIT -- flush Prints to sceen

  RETURN

GO
