-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 2005,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/tep_setup_list.sql, tbsm_base, tbsm_base31e
-- SID: 1.3
-- Date Modified: 07/11/29 10:49:25
--
-- Module Description
--   Defines the tep_setup_list stored procedure

USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tep_setup_list' AND type = 'P')
  DROP PROCEDURE tep_setup_list
GO

PRINT 'Creating procedure tep_setup_list'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/tep_setup_list.sql, tbsm_base, tbsm_base31e
-- SID: 1.3
-- Date Modified: 07/11/29 10:49:25
-- tep_setup_list is invoked from tep_setup to list the TBSM TEP Feed configuration and debuglog
CREATE PROCEDURE tep_setup_list
  @verbose TINYINT = 0
AS

  SET ANSI_DEFAULTS OFF
  SET CONCAT_NULL_YIELDS_NULL OFF
  SET NOCOUNT ON

  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT

  DECLARE @rc INT, @err INT, @sql NVARCHAR(4000), @start DATETIME

  DECLARE @soap_server ObjName, @soap_alias NVARCHAR(20)
  DECLARE @soap_uid NVARCHAR(20), @soap_pw NVARCHAR(20), @soap_port INT
  DECLARE @teps_server ObjName, @teps_type NVARCHAR(20), @teps_db NVARCHAR(20)
  DECLARE @teps_uid NVARCHAR(20), @teps_pw NVARCHAR(20), @teps_port INT, @tec_events TINYINT
  DECLARE @anchor_physical ObjName, @anchor_business ObjName
  DECLARE @linked_server NVARCHAR(100), @teps_from_clause NVARCHAR(100), @teps_provider NVARCHAR(20)
  DECLARE @enabled TINYINT, @refresh_excp_id ObjID
  DECLARE @job_name sysname, @job_id UNIQUEIDENTIFIER, @ret INT, @job_status NVARCHAR(20), @job_history NVARCHAR(4000)
  DECLARE @soap_url NVARCHAR(100), @soap_event_laststamp NVARCHAR(20)
  DECLARE @srvid INT, @srvstatus INT, @providername ObjName
  DECLARE @soap_protocol NVARCHAR(10), @teps_protocol NVARCHAR(10)
  DECLARE @db2_port_num NVARCHAR(10), @provstr NVARCHAR(1024), @port_charindex INT, @end_charindex INT, @db_charindex INT

------------------------------------------------------------------------------------
-- 
------------------------------------------------------------------------------------
  IF NOT EXISTS(SELECT * FROM tep_config)
  BEGIN
    RAISERROR('tep_setup_list:  tep_config is empty.  Use tep_setup ''ADD'' ',0,1) WITH NOWAIT
    RETURN -30000
  END

  DECLARE cur_config INSENSITIVE CURSOR FOR
    SELECT soap_server, soap_alias, soap_uid, soap_protocol, soap_port, 
           teps_server, teps_type, teps_uid, teps_protocol, teps_port,
           anchor_physical, anchor_business,
           tec_events, enabled,
           soap_url, soap_event_laststamp, teps_from_clause, refresh_excp_id
      FROM tep_config
      ORDER BY soap_server, soap_alias, teps_server
  OPEN cur_config
  FETCH NEXT FROM cur_config INTO @soap_server, @soap_alias, @soap_uid, @soap_protocol, @soap_port, 
             @teps_server, @teps_type, @teps_uid, @teps_protocol, @teps_port, 
             @anchor_physical, @anchor_business, @tec_events, @enabled,
             @soap_url, @soap_event_laststamp, @teps_from_clause, @refresh_excp_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SELECT @teps_db = CASE WHEN providername = 'IBMDADB2' THEN datasource ELSE catalog END,
           @srvid = srvid, 
           @srvstatus = srvstatus, 
           @providername = providername,
           @provstr = providerstring
      FROM master..sysservers (NOLOCK)
      WHERE srvname = SUBSTRING(@teps_from_clause,2,CHARINDEX(']',@teps_from_clause)-2)

    IF @providername = 'IBMDADB2'
    BEGIN
      SELECT @port_charindex = CHARINDEX( 'PORT=', UPPER(@provstr), 0 )
      SELECT @end_charindex  = CHARINDEX( ';', @provstr, @port_charindex )

      IF ( @port_charindex > 0 )
      BEGIN
        IF @end_charindex = 0
          SELECT @end_charindex = LEN(@provstr)
        SELECT @db2_port_num = SUBSTRING(@provstr,@port_charindex+5,@end_charindex)
      END
      ELSE
        SELECT @db2_port_num = ''

      IF ISNULL(@teps_db,'') = ''
      BEGIN
        SELECT @db_charindex = CHARINDEX( 'DATABASE=', UPPER(@provstr), 0 )
        SELECT @end_charindex  = CHARINDEX( ';', @provstr, @db_charindex )

        IF ( @db_charindex > 0 )
        BEGIN
          IF @end_charindex = 0
            SELECT @end_charindex = LEN(@provstr)
          SELECT @teps_db = SUBSTRING(@provstr,@db_charindex+9,@end_charindex-(@db_charindex+9))
        END
      END
    END
    ELSE
      SELECT @db2_port_num = ''

    RAISERROR('  @soap_server = ''%s'', @soap_alias = ''%s''',0,1,@soap_server, @soap_alias ) WITH NOWAIT
    RAISERROR('    ,@soap_uid        = ''%s''',0,1,@soap_uid   ) WITH NOWAIT
    IF ISNULL(@soap_uid,'') <> ''
      RAISERROR('    ,@soap_pw         = ''********''',0,1)        WITH NOWAIT
    ELSE
      RAISERROR('    ,@soap_pw         = ''''',0,1)        WITH NOWAIT
    IF ISNULL(@soap_protocol,'http') <> 'http' OR @verbose >= 1
      RAISERROR('    ,@soap_protocol   = ''%s''',0,1,@soap_protocol  ) WITH NOWAIT
    RAISERROR('    ,@soap_port       = %d    ',0,1,@soap_port  ) WITH NOWAIT
    RAISERROR('    ,@teps_server     = ''%s''',0,1,@teps_server) WITH NOWAIT
    RAISERROR('    ,@teps_type       = ''%s''',0,1,@teps_type  ) WITH NOWAIT
    RAISERROR('    ,@teps_db         = ''%s''',0,1,@teps_db    ) WITH NOWAIT
    IF (@db2_port_num <> '')
      RAISERROR('    ,@db2_port_num    = ''%s''',0,1,@db2_port_num) WITH NOWAIT
    RAISERROR('    ,@teps_uid        = ''%s''',0,1,@teps_uid   ) WITH NOWAIT
    RAISERROR('    ,@teps_pw         = ''********''',0,1)        WITH NOWAIT
    IF ISNULL(@teps_protocol,'http') <> 'http' OR @verbose >= 1
      RAISERROR('    ,@teps_protocol   = ''%s''',0,1,@teps_protocol  ) WITH NOWAIT
    RAISERROR('    ,@teps_port       = %d    ',0,1,@teps_port  ) WITH NOWAIT
    RAISERROR('    ,@tec_events      = %d    ',0,1,@tec_events ) WITH NOWAIT
    RAISERROR('    ,@anchor_physical = ''%s''',0,1,@anchor_physical  ) WITH NOWAIT
    RAISERROR('    ,@anchor_business = ''%s''',0,1,@anchor_business  ) WITH NOWAIT
    IF @verbose >= 1
    BEGIN
      PRINT ''
      RAISERROR('    enabled               : %-50d (internal setting listed for IBM Support)',0,1,@enabled) WITH NOWAIT
      RAISERROR('    soap_url              : %-50s (internal setting listed for IBM Support)',0,1,@soap_url) WITH NOWAIT
      RAISERROR('    soap_event_laststamp  : %-50s (internal setting listed for IBM Support)',0,1,@soap_event_laststamp) WITH NOWAIT
      RAISERROR('    refresh_excp_id       : %-50d (internal setting listed for IBM Support)',0,1,@refresh_excp_id) WITH NOWAIT
      RAISERROR('    teps_from_clause      : %-50s (internal setting listed for IBM Support)',0,1,@teps_from_clause) WITH NOWAIT
      RAISERROR('    srvid                 : %-50d (internal setting listed for IBM Support)',0,1,@srvid) WITH NOWAIT
      RAISERROR('    srvstatus             : %-50d (internal setting listed for IBM Support)',0,1,@srvstatus) WITH NOWAIT
      RAISERROR('    providername          : %-50s (internal setting listed for IBM Support)',0,1,@providername) WITH NOWAIT
      SELECT @rc = COUNT(*)
       FROM tep_objects (NOLOCK)
       WHERE soap_server = @soap_server AND soap_alias = @soap_alias
      RAISERROR('    tep_objects #         : %-50d (internal setting listed for IBM Support)',0,1,@rc) WITH NOWAIT
      SELECT @rc = COUNT(*)
        FROM tep_ims (NOLOCK)
        WHERE anchor_physical = anchor_physical
      RAISERROR('    tep_ims #             : %-50d (internal setting listed for IBM Support)',0,1,@rc) WITH NOWAIT
      SELECT @rc = COUNT(*)
        FROM tep_bsv (NOLOCK)
        WHERE teps_server = @teps_server
      RAISERROR('    tep_bsv #             : %-50d (internal setting listed for IBM Support)',0,1,@rc) WITH NOWAIT
      SELECT @rc = COUNT(*)
       FROM tep_eventdefs (NOLOCK)
         WHERE teps_server = @teps_server
      RAISERROR('    tep_eventdefs#        : %-50d (internal setting listed for IBM Support)',0,1,@rc) WITH NOWAIT
      SELECT @rc = COUNT(DISTINCT e.id)
        FROM tep_objects    AS o  (NOLOCK) 
        JOIN Exception_V           AS e  (NOLOCK) ON e._ParentCID = o.cid AND e._ParentID = o.id
        LEFT OUTER JOIN OwnedEvent AS oe (NOLOCK) ON oe.eventcid = 'EXCP' AND oe.eventid = e.id
       WHERE o.soap_server = @soap_server AND o.soap_alias = @soap_alias
         AND e.name LIKE '%[_]TEP'
         AND (e.deleted = 0 OR oe.status = 0)
      RAISERROR('    Exceptions            : %-50d (internal setting listed for IBM Support)',0,1,@rc) WITH NOWAIT
    END
    PRINT ''

    FETCH NEXT FROM cur_config INTO @soap_server, @soap_alias, @soap_uid, @soap_protocol, @soap_port, 
               @teps_server, @teps_type, @teps_uid, @teps_protocol, @teps_port, 
               @anchor_physical, @anchor_business, @tec_events, @enabled,
               @soap_url, @soap_event_laststamp, @teps_from_clause, @refresh_excp_id
  END
  DEALLOCATE cur_config

  DECLARE @job_name_len INT
  SELECT @job_name_len = MAX(LEN(j.name))
    FROM msdb..sysjobs AS j (NOLOCK)
    WHERE j.name = 'TEP Discovery'
       OR j.name LIKE 'TEP Receive Events %'
  DECLARE cur_job INSENSITIVE CURSOR FOR
    SELECT DISTINCT j.name
      FROM msdb..sysjobs AS j (NOLOCK)
      WHERE j.name = 'TEP Discovery'
         OR j.name LIKE 'TEP Receive Events %'
      ORDER BY j.name
  OPEN cur_job
  FETCH NEXT FROM cur_job INTO @job_name
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC tep_job_status @job_name=@job_name, @job_status=@job_status OUTPUT, @job_history=@job_history OUTPUT
    SET @job_name = "'"+@job_name+"'"+SPACE(@job_name_len-LEN(@job_name))
    RAISERROR('SQL TASK job %s is %s.  %s',0,1,@job_name, @job_status, @job_history) WITH NOWAIT
    FETCH NEXT FROM cur_job INTO @job_name
  END
  DEALLOCATE cur_job
  PRINT ''

  SELECT @rc = COUNT(DISTINCT e.id)
    FROM Exception_V AS e (NOLOCK)
    LEFT OUTER JOIN OwnedEvent AS oe (NOLOCK) ON oe.eventcid = 'EXCP' AND oe.eventid = e.id
   WHERE e.name LIKE '%[_]TEP'
     AND (e.deleted = 0 OR oe.status = 0)
  RAISERROR('%d current TEP Exceptions',0,1,@rc) WITH NOWAIT
  IF @rc > 0
  BEGIN
    IF @rc > 100
      RAISERROR('Showing most recent 100 Exceptions',0,1,@rc) WITH NOWAIT
    SET ROWCOUNT 100
      SELECT e._ParentCID, e._ParentID, oe.noteid, e.id AS 'excpid',
             e._AlertState, LEFT(e._SEQ_NO,30) AS 'SEQ_NO',
             e.name, e._OBJECT_NAME, REPLACE(e.description,CHAR(10),' \n '),
             e.ctime
        FROM Exception_V AS e (NOLOCK)
        LEFT OUTER JOIN OwnedEvent AS oe (NOLOCK) ON oe.eventcid = 'EXCP' AND oe.eventid = e.id
        WHERE e.name LIKE '%[_]TEP'
          AND (e.deleted = 0 OR oe.status = 0)
        ORDER BY e.id DESC
    SET ROWCOUNT 0
  END
  PRINT ''

  IF @verbose >= 2
  BEGIN
    SET NOCOUNT OFF
    PRINT 'SELECT * FROM tep_objects'
           SELECT * FROM tep_objects
    PRINT 'SELECT * FROM tep_ims'
           SELECT * FROM tep_ims
    PRINT 'SELECT * FROM tep_bsv'
           SELECT * FROM tep_bsv
    PRINT 'SELECT * FROM tep_discmap'
           SELECT * FROM tep_discmap
    PRINT 'SELECT * FROM tep_eventdefs'
           SELECT * FROM tep_eventdefs
    SET NOCOUNT ON
  END

  EXEC tep_log @action='LIST', @maxrows=5000


GO